@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -64,7 +64,8 @@ void AI::Play( void ) {
   // set up progress indicator; number of steps is unit count plus 3
   // for objectives identification, objectives assignment, and production
   progress = new ProgressWindow( 0, 0, view->Width()/2, 30,
-                                 1, 3 + player->Units(0), WIN_CENTER, view );
+                                 1, 3 + player->Units(0), NULL,
+                                 WIN_CENTER, view );
 
   IdentifyObjectives();
   progress->Advance( 1 );
@@ -488,7 +489,7 @@ void AI::CommandUnitDefend( Unit *u, AIObj &obj ) {
                 SameDirection( u->Position(), tg->Position(), obj.pos )) ) {
         CommandUnitAttack( u, obj );
     } else if ( obj.pos == u->Position() ) CommandUnitAttack( u, obj );
-  } 
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -597,6 +598,9 @@ bool AI::CommandUnitRepair( Unit *u ) {
 
       if ( u->Position() == b->Position() ) {
         b->SetCrystals( b->Crystals() - CRYSTALS_REPAIR );
+        if ( mission.GetHistory() )
+          mission.GetHistory()->RecordUnitEvent( *u, History::HIST_UEVENT_REPAIR );
+
         u->Repair();
       }
       return true;
