@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 // RETURNS    : -1 on error, owner identifier (>= 0) otherwise
 ////////////////////////////////////////////////////////////////////////
 
-short Building::Load( File &file ) {
+short Building::Load( MemBuffer &file ) {
   unsigned char pid, minw, maxw;
 
   b_id = file.Read16();
@@ -57,7 +57,7 @@ short Building::Load( File &file ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Building::Save( File &file ) const {
+int Building::Save( MemBuffer &file ) const {
   unsigned char pid;
 
   if ( b_player ) pid = b_player->ID();
