@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,8 +30,8 @@ class Building : public UnitContainer, public Node, public MapObject {
 public:
   Building( void ) :
     MapObject(MO_BUILDING), b_name(0), b_player(0) {}
-  short Load( File &file );
-  int Save( File &file ) const;
+  short Load( MemBuffer &file );
+  int Save( MemBuffer &file ) const;
 
   bool Allow( const Unit *unit ) const;
 
