@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ Combat::Combat( Unit *att, Unit *def ) :
 // RETURNS    : -1 on error, 0 on success
 ////////////////////////////////////////////////////////////////////////
 
-int Combat::Load( File &file, Mission &mission ) {
+int Combat::Load( MemBuffer &file, Mission &mission ) {
   aamod = admod = damod = ddmod = 0;
   c_att = mission.GetUnit( file.Read16() );
   c_def = mission.GetUnit( file.Read16() );
@@ -59,7 +59,7 @@ int Combat::Load( File &file, Mission &mission ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Combat::Save( File &file ) const {
+int Combat::Save( MemBuffer &file ) const {
   file.Write16( c_att->ID() );
   file.Write16( c_def->ID() );
   return 0;
@@ -143,9 +143,36 @@ void Combat::CalcModifiers( const Map &map ) {
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : Combat::Resolve
-// DESCRIPTION: Pop up a window displaying the raging firefight between
-//              two units.
+// NAME       : Combat::CalcResults
+// DESCRIPTION: Resolve a firefight between two units with a known
+//              outcome.
+// PARAMETERS : atthits - hits scored by attacker
+//              defhits - hits scored by defender
+// RETURNS    : Point containing hits by the attacker and defender
+////////////////////////////////////////////////////////////////////////
+
+Point Combat::CalcResults( unsigned char atthits, unsigned char defhits ) const {
+  unsigned short numatts = c_att->GroupSize(), numdefs = c_def->GroupSize();
+
+  c_def->Hit( atthits );
+  c_att->Hit( defhits );
+
+  // award experience: 1 if the enemy troops were reduced, 3 if they were destroyed
+  if ( !c_att->IsMine() ) {
+    if ( !c_att->IsAlive() ) c_def->AwardXP( 3 );
+    else if ( c_att->GroupSize() < numatts ) c_def->AwardXP( 1 );
+  }
+  if ( !c_def->IsMine() ) {
+    if ( !c_def->IsAlive() ) c_att->AwardXP( 3 );
+    else if ( c_def->GroupSize() < numdefs ) c_att->AwardXP( 1 );
+  }
+
+  return Point( atthits, defhits );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : Combat::CalcResults
+// DESCRIPTION: Resolve a firefight between two units.
 // PARAMETERS : -
 // RETURNS    : Point containing hits by the attacker and defender
 //
@@ -159,73 +186,50 @@ Point Combat::CalcResults( void ) {
   Point p1 = c_att->Position(), p2 = c_def->Position();
   unsigned short numatts = c_att->GroupSize(), numdefs = c_def->GroupSize(),
     dist = Distance( p1.x, p1.y, p2.x, p2.y );
-  unsigned char ahits, dhits;
+  int i;
+  unsigned char ahits = 0, dhits = 0, atohit, dtohit, atodef, dtodef,
+                aapool = 0, adpool = 0, dapool = 0, ddpool = 0,
+                aastr, adstr, dastr, ddstr,
+                axp = c_att->XPLevel() * 2, dxp = c_def->XPLevel() * 2;
+  atohit = MAX( 25 + axp - (dist - 1) * 2 + aamod, 1 );
+  atodef = 25 + axp + admod;
+  dtodef = 25 + dxp + ddmod;
+  aastr = c_att->OffensiveStrength( c_def ) + axp;
+  adstr = c_att->DefensiveStrength() + axp;
+  dastr = c_def->OffensiveStrength( c_att ) + dxp;
+  ddstr = c_def->DefensiveStrength() + dxp;
 
-  if ( !c_att->IsDummy() ) {
-    int i;
-    unsigned char atohit, dtohit, atodef, dtodef,
-                  aapool = 0, adpool = 0, dapool = 0, ddpool = 0,
-                  aastr, adstr, dastr, ddstr,
-                  axp = c_att->XPLevel() * 2, dxp = c_def->XPLevel() * 2;
-    ahits = dhits = 0;
-    atohit = MAX( 25 + axp - (dist - 1) * 2 + aamod, 1 );
-    atodef = 25 + axp + admod;
-    dtodef = 25 + dxp + ddmod;
-    aastr = c_att->OffensiveStrength( c_def ) + axp;
-    adstr = c_att->DefensiveStrength() + axp;
-    dastr = c_def->OffensiveStrength( c_att ) + dxp;
-    ddstr = c_def->DefensiveStrength() + dxp;
+  // can the defender return fire?
+  if ( (dist == 1) && c_def->CanHit( c_att ) ) {
+    dtohit = 25 + dxp + damod;
+  } else dtohit = 0;
 
-    // can the defender return fire?
-    if ( (dist == 1) && c_def->CanHit( c_att ) ) {
-      dtohit = 25 + dxp + damod;
-    } else dtohit = 0;
-
-    for ( i = 0; i < numatts; ++i ) {
-      if ( random( 1, 100 ) <= atohit )
-         aapool += aastr * random( 80, 120 ) / 100;
-      if ( random( 1, 100 ) <= atodef )
-         adpool += adstr * random( 80, 120 ) / 100;
-    }
-
-    for ( i = 0; i < numdefs; ++i ) {
-      if ( random( 1, 100 ) <= dtohit )
-        dapool += dastr * random( 80, 120 ) / 100;
-      if ( random( 1, 100 ) <= dtodef )
-        ddpool += ddstr * random( 80, 120 ) / 100;
-    }
-
-    // set defence pools to a minimum of the enemy unit strength;
-    // that avoids division by zero and leads to somewhat sane results
-    if ( adpool <= dastr ) adpool = MAX( 1, dastr );
-    if ( ddpool < aastr ) ddpool = aastr;   // attacker can't have 0 strength
-
-    // generally, the number of hits is determined by comparing the attack and
-    // defence pool values. To reduce the impact of luck, however, we take the
-    // average of the "randomized" and a strictly deterministic number.
-    ahits = (aapool/ddpool +
-            (aastr * atohit)/(ddstr * dtodef) + 1) / 2;
-    dhits = (dapool/adpool +
-            (dastr * dtohit)/(adstr * atodef) + 1) / 2;
-  } else {
-    // we only have dummy units; the modifiers tell us about the casualties
-    ahits = aamod;
-    dhits = ddmod;
+  for ( i = 0; i < numatts; ++i ) {
+    if ( random( 1, 100 ) <= atohit )
+      aapool += aastr * random( 80, 120 ) / 100;
+    if ( random( 1, 100 ) <= atodef )
+      adpool += adstr * random( 80, 120 ) / 100;
   }
 
-  c_def->Hit( ahits );
-  c_att->Hit( dhits );
-
-  // award experience: 1 if the enemy troops were reduced, 3 if they were destroyed
-  if ( !c_att->IsMine() ) {
-    if ( !c_att->IsAlive() ) c_def->AwardXP( 3 );
-    else if ( c_att->GroupSize() < numatts ) c_def->AwardXP( 1 );
-  }
-  if ( !c_def->IsMine() ) {
-    if ( !c_def->IsAlive() ) c_att->AwardXP( 3 );
-    else if ( c_def->GroupSize() < numdefs ) c_att->AwardXP( 1 );
+  for ( i = 0; i < numdefs; ++i ) {
+    if ( random( 1, 100 ) <= dtohit )
+      dapool += dastr * random( 80, 120 ) / 100;
+    if ( random( 1, 100 ) <= dtodef )
+      ddpool += ddstr * random( 80, 120 ) / 100;
   }
 
-  return Point( ahits, dhits );
+  // set defence pools to a minimum of the enemy unit strength;
+  // that avoids division by zero and leads to somewhat sane results
+  if ( adpool <= dastr ) adpool = MAX( 1, dastr );
+  if ( ddpool < aastr ) ddpool = aastr;   // attacker can't have 0 strength
+
+  // generally, the number of hits is determined by comparing the attack and
+  // defence pool values. To reduce the impact of luck, however, we take the
+  // average of the "randomized" and a strictly deterministic number.
+  ahits = (aapool/ddpool +
+          (aastr * atohit)/(ddstr * dtodef) + 1) / 2;
+  dhits = (dapool/adpool +
+          (dastr * dtohit)/(adstr * atodef) + 1) / 2;
+
+  return CalcResults( ahits, dhits );
 }
-
