@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -35,18 +35,17 @@ class Mission;
 
 class Combat : public Node {
 public:
-  friend class History;
-
   Combat( void ) {}
   Combat( Unit *att, Unit *def );
-  int Load( File &file, Mission &mission );
-  int Save( File &file ) const;
+  int Load( MemBuffer &file, Mission &mission );
+  int Save( MemBuffer &file ) const;
 
   Unit *GetAttacker( void ) const { return c_att; }
   Unit *GetDefender( void ) const { return c_def; }
 
   void CalcModifiers( const Map &map );
   Point CalcResults( void );
+  Point CalcResults( unsigned char atthits, unsigned char defhist ) const;
 
 private:
   Unit *c_att;       // attacker
