@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -77,8 +77,8 @@ public:
   Transport( const UnitType *type, Player *player, unsigned short id,
              const Point &pos );
 
-  int Load( File &file, const UnitType *type, Player *player );
-  int Save( File &file ) const;
+  int Load( MemBuffer &file, const UnitType *type, Player *player );
+  int Save( MemBuffer &file ) const;
 
   void SetCrystals( unsigned short crystals );
   unsigned short Crystals( void ) const { return t_crystals; }
