@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 // RETURNS    : identifier of the event owner, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-short Event::Load( File &file ) {
+short Event::Load( MemBuffer &file ) {
   int i;
 
   e_id = file.Read8();
@@ -56,7 +56,7 @@ short Event::Load( File &file ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Event::Save( File &file ) const {
+int Event::Save( MemBuffer &file ) const {
   int i;
 
   file.Write8( e_id );
@@ -86,7 +86,7 @@ int Event::Save( File &file ) const {
 
 bool Event::Check( void ) {
   bool rc = CheckTrigger();
- 
+
   if ( rc ) {
     TLWList deps;
     deps.AddHead( new TLWNode( "", this, ID() ) );
@@ -301,6 +301,27 @@ void Event::Execute( View *view ) {
       }
     }
     break; }
+  case EVENT_DESTROY_UNIT: {
+    Unit *u = mis->GetUnit( e_data[0] );
+    if ( u && ((e_data[1] == -1) ||
+        (u->Owner() && (e_data[1] == u->Owner()->ID())))) {
+      Point p = u->Position();
+      bool show = !u->IsSheltered() && mv->Enabled();
+
+      if ( show ) mwin->DisplayHex( p );
+
+      if ( mis->GetHistory() )
+        mis->GetHistory()->RecordUnitEvent( *u, History::HIST_UEVENT_DESTROY );
+
+      u->Hit( MAX_GROUP_SIZE );
+      mis->GetMap().SetUnit( 0, p );
+
+      if ( show ) {
+        mwin->FadeOutUnit( u->Image(), p );
+        mwin->Show( mv->UpdateHex( p ) );
+      }
+    } else show_msg = false;
+    break; }
   case EVENT_MANIPULATE_EVENT: {
     Event *e = mis->GetEvent( e_data[0] );
     if ( e ) {
@@ -390,7 +411,7 @@ void Event::Execute( View *view ) {
 
 void Event::DisplayMessage( Player *p, short msg, short title, Mission *m,
                             bool show, View *view ) const {
-  if ( (msg == -1) || !p->IsHuman() ) show = false;
+  if ( (msg == -1) || !p->IsInteractive() ) show = false;
 
   if ( show ) {
     MapWindow *mwin = Gam->GetMapWindow();
