@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,31 +39,74 @@
 extern Options CFOptions;
 
 // button identifiers used by the game hook function dispatcher
-#define G_BUTTON_END_TURN	 10
-#define G_BUTTON_MAP		 11
-#define G_BUTTON_BRIEFING	 12
-#define G_BUTTON_SAVE		 13
-#define G_BUTTON_LEV_INFO	 14
-#define G_BUTTON_GENERAL_OPTIONS 15
-#define G_BUTTON_LANGUAGE_OPTIONS 16
-#define G_BUTTON_VIDEO_OPTIONS	 17
-#define G_BUTTON_SOUND_OPTIONS	 18
-#define G_BUTTON_ABORT		 19
-#define G_BUTTON_QUIT		 20
+enum {
+  G_BUTTON_END_TURN = 10,
+  G_BUTTON_MAP,
+  G_BUTTON_BRIEFING,
+  G_BUTTON_SAVE,
+  G_BUTTON_LEV_INFO,
+  G_BUTTON_GENERAL_OPTIONS,
+  G_BUTTON_LANGUAGE_OPTIONS,
+  G_BUTTON_VIDEO_OPTIONS,
+  G_BUTTON_SOUND_OPTIONS,
+  G_BUTTON_KEYBOARD_OPTIONS,
+  G_BUTTON_ABORT,
+  G_BUTTON_QUIT,
 
-#define G_BUTTON_SHUTDOWN	 21
+  G_BUTTON_SHUTDOWN,
 
-#define G_BUTTON_UNIT_INFO	 22
-#define G_BUTTON_UNIT_CONTENT	 23
-#define G_BUTTON_UNIT_SWEEP	 24
-#define G_BUTTON_UNIT_UNDO	 25
+  G_BUTTON_UNIT_INFO,
+  G_BUTTON_UNIT_CONTENT,
+  G_BUTTON_UNIT_SWEEP,
+  G_BUTTON_UNIT_UNDO,
+  G_BUTTON_MINIMIZE_WINDOW
+};
 
-#define G_KEY_QUIT		0
-#define G_KEY_INFO		1
-#define G_KEY_MAP		2
-#define G_KEY_END_TURN		3
-#define G_KEY_CONTENT		4
-#define G_KEY_SWEEP		5
+enum {
+  G_KEY_QUIT = 0,
+  G_KEY_INFO,
+  G_KEY_MAP,
+  G_KEY_END_TURN,
+  G_KEY_CONTENT,
+  G_KEY_SWEEP
+};
+
+#ifndef DISABLE_NETWORK
+class NetworkProgressWindow : public ProgressWindow {
+public:
+  NetworkProgressWindow( unsigned short w, unsigned short h,
+                         const char *msg, View *view );
+
+  bool Cancelled( void );
+};
+
+NetworkProgressWindow::NetworkProgressWindow( unsigned short w,
+                       unsigned short h, const char *msg, View *view ) :
+       ProgressWindow( 0, 0, w, h, 0, 100, msg,
+                       WIN_CENTER|WIN_PROG_ABORT, view ) {}
+
+bool NetworkProgressWindow::Cancelled( void ) {
+  if ( Get() < 100 ) Advance( 1 );
+  else Set( 0 );
+
+  bool cancel = ProgressWindow::Cancelled();
+
+  if ( cancel ) {
+    // ask for confirmation
+    string buttons;
+    buttons.append( MSG(MSG_B_YES) );
+    buttons += '|';
+    buttons.append( MSG(MSG_B_NO) );
+    DialogWindow *req = new DialogWindow( NULL,
+        MSG(MSG_ASK_ABORT_NETWORK), buttons, 1, 0, view );
+    req->SetButtonID( 1, 0 );
+    cancel = (req->EventLoop() == GUI_CLOSE);
+    view->CloseWindow( req );
+  }
+
+  return cancel;
+}
+#endif
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Game::Game
@@ -72,14 +115,12 @@ extern Options CFOptions;
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-Game::Game( View *view ) {
-  this->view = view;
-  mwin = NULL;
-  mission = NULL;
-  unit = NULL;
-  shader = NULL;
-
+Game::Game( View *view ) : mission(0), mwin(0), unit(0), shader(0), view(view) {
   InitKeys();
+
+#ifndef DISABLE_NETWORK
+  peer = NULL;
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -93,6 +134,10 @@ Game::~Game( void ) {
   if ( mwin ) view->CloseWindow( mwin );
   delete mission;
   delete shader;
+
+#ifndef DISABLE_NETWORK
+  delete peer;
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -126,7 +171,7 @@ void Game::InitWindows( void ) {
   if ( mission ) {
     mwin = new MapWindow( 0, 0, view->Width(), view->Height(), 0, view );
     if ( CFOptions.GetDamageIndicator() ) mwin->GetMapView()->EnableUnitStats();
-    mwin->GetMapView()->SetMap( &mission->GetMap() );	// attach map to map window
+    mwin->GetMapView()->SetMap( &mission->GetMap() );  // attach map to map window
     shader = new MoveShader( &mission->GetMap(), mission->GetUnits(),
                              mwin->GetMapView()->GetFogBuffer() );
     ExecPreStartEvents();
@@ -247,7 +292,7 @@ string Game::CreateSaveFileName( const char *filename ) const {
 int Game::Save( const char *filename ) {
   string fname( CreateSaveFileName( filename ) );
   if ( fname.length() == 0 ) return -1;
- 
+
   File file( fname );
   if ( !file.Open( "wb", false ) ) {
     new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_WRITE), 0, view );
@@ -273,7 +318,6 @@ int Game::Save( const char *filename ) {
   return 0;
 }
 
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Game::Load
 // DESCRIPTION: Load a game from a mission file (start a new game) or
@@ -283,30 +327,60 @@ int Game::Save( const char *filename ) {
 ////////////////////////////////////////////////////////////////////////
 
 int Game::Load( const char *filename ) {
-  int rc = -1;
   File file( filename );
   if ( !file.Open( "rb" ) ) return -1;
 
-  mission = new Mission();
-  if ( mission->Load( file ) != -1 ) {
-
+  int rc = Load( file );
+  if ( rc != -1 ) {
     // set last filename
     last_file_name = file_part( filename );
     unsigned int i = last_file_name.rfind( '.' );
     last_file_name.erase( i );
+  } else {
+    cerr << "Error loading " << filename << endl;
+  }
+
+  return rc;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : Game::Load
+// DESCRIPTION: Load a game from a buffer.
+// PARAMETERS : buffer - data buffer
+// RETURNS    : 0 on success, -1 otherwise
+////////////////////////////////////////////////////////////////////////
+
+int Game::Load( MemBuffer &buffer ) {
+  int rc = -1;
+
+  mission = new Mission();
+  if ( mission->Load( buffer ) != -1 ) {
 
     mission->SetLocale( CFOptions.GetLanguage() );
 
     unsigned short flags = mission->GetFlags();
-    // if this is a new map (GI_SAVEFILE not set) GI_SKIRMISH,
-    // GI_CAMPAIGN, and GI_AI only serve as information
     if ( !(flags & GI_SAVEFILE) ) {
+      // if this is a new map (GI_SAVEFILE not set) GI_SKIRMISH,
+      // GI_CAMPAIGN, and GI_AI only serve as information
       flags &= ~(GI_AI|GI_CAMPAIGN|GI_SKIRMISH);
       flags |= ((CFOptions.IsPBEM() ? GI_PBEM : 0)|
+               (CFOptions.IsNetwork() ? GI_NETWORK : 0)|
+               (CFOptions.IsAI() ? GI_AI : 0)|
                (CFOptions.GetCampaign() ? GI_CAMPAIGN : 0));
+      mission->SetFlags( flags );
     }
-    mission->SetFlags( flags );
+
     rc = 0;
+
+#ifdef DISABLE_NETWORK
+    if ( flags & GI_NETWORK ) {
+      cerr << "Error: " << PROGRAMNAME << " was compiled without networking support" << endl;
+      rc = -1;
+
+      delete mission;
+      mission = NULL;
+    }
+#endif
   } else {
     delete mission;
     mission = NULL;
@@ -360,6 +434,7 @@ int Game::SwitchMap( const char *sequel ) {
 ////////////////////////////////////////////////////////////////////////
 
 GUI_Status Game::StartTurn( void ) {
+  GUI_Status rc = GUI_OK;
   MapView *mv = mwin->GetMapView();
   Player &player = mission->GetPlayer();
 
@@ -369,7 +444,7 @@ GUI_Status Game::StartTurn( void ) {
   view->SetFGPen( player.LightColor() );   // set player color scheme
   view->SetBGPen( player.DarkColor() );
 
-  if ( player.IsHuman() ) {
+  if ( player.IsInteractive() ) {
     NoteWindow *nw;
 
     // show turn information and greeting dialog
@@ -380,15 +455,15 @@ GUI_Status Game::StartTurn( void ) {
     nw = new NoteWindow( player.Name(), turnmsg, WIN_FONT_BIG|WIN_CENTER, view );
     nw->EventLoop();
     view->CloseWindow( nw );
-  }
 
-  // in email games set up or check player passwords
-  if ( mission->GetFlags() & GI_PBEM ) {
-    if ( !player.Password() ) {
-      if ( !SetPlayerPassword( &player ) ) return GUI_RESTART;
-    } else if ( !CheckPassword( player.Name(), MSG(MSG_ENTER_PASSWORD),
-                player.Password(), 2 ) )
-      return GUI_RESTART;
+    // in email games set up or check player passwords
+    if ( mission->GetFlags() & GI_PBEM ) {
+      if ( !player.Password() ) {
+        if ( !SetPlayerPassword( &player ) ) return GUI_RESTART;
+      } else if ( !CheckPassword( player.Name(), MSG(MSG_ENTER_PASSWORD),
+                  player.Password(), 2 ) )
+        return GUI_RESTART;
+    }
   }
 
   undo.Disable();
@@ -397,10 +472,9 @@ GUI_Status Game::StartTurn( void ) {
     // replay
     History *history = mission->GetHistory();
     if ( history ) {
-      history->Replay( mwin );
+      if ( player.IsInteractive() ) history->Replay( mwin );
       mission->SetHistory( NULL );
       delete history;
-      history = NULL;
     }
 
     // begin new turn
@@ -422,7 +496,37 @@ GUI_Status Game::StartTurn( void ) {
     CheckEvents();
     AI ai( *mission );
     ai.Play();
-    EndTurn();
+    rc = EndTurn();
+
+#ifndef DISABLE_NETWORK
+  } else if ( player.IsRemote() ) {
+    // show message while waiting
+    NetworkProgressWindow *pw = new NetworkProgressWindow(
+        view->Width() * 2 / 3, view->SmallFont()->Height() + 20,
+        MSG(MSG_NET_WAITING), view );
+    DynBuffer *updates = peer->Receive( pw );
+    if ( updates ) {
+      History remote;
+      remote.Load( *updates, *mission );
+
+      CheckEvents();
+      Execute( remote );
+    } else {
+      NoteWindow *ew = new NoteWindow( MSG(MSG_ERROR),
+          MSG(MSG_ERR_NETWORK), WIN_FONT_BIG|WIN_CENTER, view );
+      ew->EventLoop();
+      view->CloseWindow( ew );
+      return GUI_RESTART;
+    }
+
+    view->CloseWindow( pw );
+
+    // make some noise so the local player know it's his turn now
+    Audio::PlaySfx( Audio::SND_GUI_ASK, 0 );
+
+    rc = EndTurn();
+#endif
+
   } else {
 
     mv->Enable();
@@ -446,7 +550,7 @@ GUI_Status Game::StartTurn( void ) {
     view->EnableUpdates();
     view->Refresh();
   }
-  return GUI_OK;
+  return rc;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -685,6 +789,23 @@ GUI_Status Game::EndTurn( void ) {
         nw->SetButtonID( 0, GUI_RESTART );
       }
     } else {
+
+#ifndef DISABLE_NETWORK
+      if ( p.IsRemote() ) {
+        // send data to peer
+        History *history = mission->GetHistory();
+
+        if ( !history ) {
+          cerr << "Error: No history available in network game" << endl;
+          return GUI_ERROR;
+        } else {
+          DynBuffer buf;
+          history->Save( buf, true );
+          peer->Send( buf );
+        }
+      }
+#endif
+
       mwin->GetMapView()->Disable();
       mwin->Draw();
       mwin->Show();
@@ -700,11 +821,14 @@ GUI_Status Game::EndTurn( void ) {
 // NAME       : Game::ResolveBattle
 // DESCRIPTION: Calculate and display battle outcomes for all clashes
 //              queued.
-// PARAMETERS : com - battle to resolve
+// PARAMETERS : com    - battle to resolve
+//              result - optional precalculated battle results (hits
+//                       scored by attacker/defender, NULL to calculate)
+//
 // RETURNS    : GUI status
 ////////////////////////////////////////////////////////////////////////
 
-void Game::ResolveBattle( Combat *com ) {
+void Game::ResolveBattle( Combat *com, const Point *result /* = NULL */ ) {
   Unit *att = com->GetAttacker();
   Unit *def = com->GetDefender();
   if ( !att->IsAlive() || !def->IsAlive() ) return;
@@ -712,20 +836,25 @@ void Game::ResolveBattle( Combat *com ) {
   // calculate modifiers for combat
   Map &map = mission->GetMap();
   History *hist = mission->GetHistory();
-  CombatWindow *cwin = 0;
+  CombatWindow *cwin = NULL;
 
-  if ( mission->GetPlayer().IsHuman() )
+  if ( mission->GetPlayer().IsInteractive() )
     cwin = new CombatWindow( com, mwin, view );
 
   Point apos( att->Position() ), dpos( def->Position() );
-  Point hits( com->CalcResults() );
+  Point hits;
+
+  if ( result == NULL )
+    hits = com->CalcResults();
+  else
+    hits = com->CalcResults( result->x, result->y );
 
   // record as a combat event
   if ( hist && !att->IsDummy() )
     hist->RecordCombatEvent( *com, hits.y, hits.x );
 
-  if ( !att->IsAlive() ) map.SetUnit( 0, apos );
-  if ( !def->IsAlive() ) map.SetUnit( 0, dpos );
+  if ( !att->IsAlive() ) map.SetUnit( NULL, apos );
+  if ( !def->IsAlive() ) map.SetUnit( NULL, dpos );
 
   if ( cwin ) {
     cwin->Draw();
@@ -822,7 +951,8 @@ Unit *Game::MoveUnit( Unit *u, const Point &dest ) {
         else parent = static_cast<Building *>(pobj);
         UnitLoadWindow *ulw = new UnitLoadWindow(
             *static_cast<Transport *>(u), *parent,
-            *map->GetUnitSet(), *map->GetTerrainSet(), view );
+            *map->GetUnitSet(), *map->GetTerrainSet(),
+            mission->GetHistory(), view );
 
         bool aborted = false;
         if ( ulw->Opened() ) aborted = (ulw->EventLoop() == GUI_CLOSE);
@@ -848,25 +978,17 @@ Unit *Game::MoveUnit( Unit *u, const Point &dest ) {
     undo.Register( u );
     RemoveUnit( u );
 
-    short step, oldhex = map->HexTypeID( dest ); 
+    short step;
     while ( (step = path.GetStep( u->Position() )) != -1)
       MoveUnit( u, (Direction)step );
 
-    u->SetFlags( U_MOVED );
-
     if ( sfx ) sfx->Stop();
 
-    const Point &posnew = u->Position();
-    int conquer = map->SetUnit( u, posnew );
-    mwin->Show( mv->UpdateHex( posnew ) );
+    EndMovement( u );
 
-    if ( conquer == 1 ) {              // a building was conquered
-      History *h = mission->GetHistory();
-      if ( h ) h->RecordTileEvent( map->HexTypeID( posnew ), oldhex, posnew.x, posnew.y );
-    }
+    mwin->Show( mv->UpdateHex( u->Position() ) );
 
-    if ( u->IsSlow() || !UnitTargets( u ) ) {
-      u->SetFlags( U_DONE );
+    if ( !u->IsReady() ) {
       if ( u == unit ) DeselectUnit();
       CheckEvents();
       return NULL;
@@ -911,6 +1033,31 @@ int Game::MoveUnit( Unit *u, Direction dir ) {
   return 0;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : Game::EndMovement
+// DESCRIPTION: Finalize a unit move.
+// PARAMETERS : u - unit to be moved
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void Game::EndMovement( Unit *u ) const {
+  Map &map = mission->GetMap();
+
+  u->SetFlags( U_MOVED );
+
+  const Point &pos = u->Position();
+  short oldhex = map.HexTypeID( pos );
+  int conquer = map.SetUnit( u, pos );
+
+  if ( conquer == 1 ) {              // a building was conquered
+    History *h = mission->GetHistory();
+    if ( h ) h->RecordTileEvent( map.HexTypeID( pos ), oldhex, pos.x, pos.y );
+  }
+
+  if ( u->IsSlow() || !UnitTargets( u ) )
+    u->SetFlags( U_DONE );
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Game::RemoveUnit
 // DESCRIPTION: Remove a unit from the map.
@@ -1171,7 +1318,7 @@ void Game::Undo( void ) {
   mwin->Show();
 
   if ( mission->GetHistory() )
-    mission->GetHistory()->EraseMoveEvents( *u );
+    mission->GetHistory()->UndoMove( *u );
 
   undo.Disable();
 }
@@ -1191,36 +1338,61 @@ GUI_Status Game::HandleEvent( const SDL_Event &event ) {
 
   // check for keyboard commands
   if ( event.type == SDL_KEYDOWN ) {
+    // some SDL ports return illegal key values
+    short key = event.key.keysym.sym & 0x1ff;
+    const SDLKey *keymap = CFOptions.GetKeyBindings();
 
-    switch ( event.key.keysym.sym ) {
+    // map user-defined keys to standard ones
+    if ((key > SDLK_FIRST) && (key < SDLK_LAST)) {
+      // KEYBIND_MINIMIZE is a global binding that is not specific
+      // to the Game class, so it isn't handled here
+
+      if (key == keymap[KEYBIND_END_TURN]) key = keys[G_KEY_MAP];
+      else if (key == keymap[KEYBIND_SHOW_MAP]) key = keys[G_KEY_MAP];
+      else if (key == keymap[KEYBIND_UNIT_CONTENT]) key = keys[G_KEY_CONTENT];
+      else if (key == keymap[KEYBIND_UNIT_INFO]) key = keys[G_KEY_INFO];
+      else if (key == keymap[KEYBIND_UNIT_SWEEP]) key = keys[G_KEY_SWEEP];
+      else if (key == keymap[KEYBIND_UNIT_SELECT]) {
+        // select the unit underneath the cursor
+        SelectCommand( mv->Cursor() );
+        key = SDLK_UNKNOWN;
+      } else if (key == keymap[KEYBIND_GAME_MENU]) {
+        GameMenu();
+        key = SDLK_UNKNOWN;
+      } else if (key == keymap[KEYBIND_UNIT_MENU]) {
+        Unit *u = map.GetUnit( mv->Cursor() );
+        if ( u ) UnitMenu( u );
+        key = SDLK_UNKNOWN;
+      } else if (key == keymap[KEYBIND_UNIT_UNDO]) {
+        if ( undo.GetUnit() != NULL) Undo();
+        key = SDLK_UNKNOWN;
+      } else if (key == keymap[KEYBIND_UNIT_NEXT]) {
+        SelectNextUnit();
+        key = SDLK_UNKNOWN;
+      }
+    }
+
+    switch ( key ) {
     case SDLK_KP1: case SDLK_KP2: case SDLK_KP3:
     case SDLK_KP4: case SDLK_KP6: case SDLK_KP7:
     case SDLK_KP8: case SDLK_KP9:
       if ( !(event.key.keysym.mod & KMOD_NUM) ) {
-        ScrollCommand( event.key.keysym.sym );
+        ScrollCommand( key );
         break;
       }
       // fall through
     case SDLK_LEFT: case SDLK_RIGHT: case SDLK_UP: case SDLK_DOWN:
-      MoveCommand( event.key.keysym.sym );
-      break;
-    case SDLK_SPACE:      // select the unit underneath the cursor
-      SelectCommand( mv->Cursor() );
-      break;
-    case SDLK_F2:         // select next unit
-      SelectNextUnit();
+      MoveCommand( key );
       break;
     case SDLK_ESCAPE:
       if ( unit ) {
         DeselectUnit();
         break;
-      }                  // fall through...
-    case SDLK_F1:        // game menu
+      }
       GameMenu();
       break;
     default:
       // these shortcuts change with the locale
-      short key = event.key.keysym.sym;
 
       if ( key == keys[G_KEY_INFO] ) {
         Unit *u = map.GetUnit( mv->Cursor() );
@@ -1230,7 +1402,7 @@ GUI_Status Game::HandleEvent( const SDL_Event &event ) {
       } else if ( key == keys[G_KEY_CONTENT] ) {
         MapObject *mo = map.GetMapObject( mv->Cursor() );
         if ( mo ) {
-          if ( mo->IsShop() ) 
+          if ( mo->IsShop() )
             ContainerContent( static_cast<UnitContainer *>(
                               static_cast<Building *>(mo)) );
           else if ( static_cast<Unit *>(mo)->IsTransport() )
@@ -1258,7 +1430,7 @@ GUI_Status Game::HandleEvent( const SDL_Event &event ) {
       if ( event.button.button == SDL_BUTTON_RIGHT ) {
         if ( u ) UnitMenu( u );
         else GameMenu();
-      } else {		// middle mouse button
+      } else {    // middle mouse button
         if ( u ) {
           if ( u->IsTransport() ) ContainerContent( static_cast<Transport *>(u) );
         } else if ( map.GetBuilding( pos ) )
@@ -1487,7 +1659,7 @@ void Game::ShowBriefing( void ) const {
 GUI_Status Game::ShowDebriefing( Player &player, bool restart ) {
   GUI_Status rc = GUI_OK;
 
-  if ( player.IsHuman() ) {
+  if ( player.IsInteractive() ) {
     Player *winner = NULL;
     bool draw = false;
     const char *msg;
@@ -1531,10 +1703,140 @@ GUI_Status Game::ShowDebriefing( Player &player, bool restart ) {
       note->EventLoop();
       view->CloseWindow( note );
     }
-  }
+  } else if ( restart ) rc = GUI_RESTART;
+
   return rc;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : Game::Execute
+// DESCRIPTION: Execute the events from a recorded turn history session.
+//              In contrast to History::Replay() which only creates fake
+//              actions for display, this function directly affects the
+//              current state of the game, ie. all actions are treated
+//              exactly as if they had been triggered by the current
+//              player.
+// PARAMETERS : history - recorded session to execute
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void Game::Execute( const History &history ) {
+  const List &events = history.GetEvents();
+
+  // many event types can simply be ignored;
+  // all we care for is movement, attacks, plus (user-triggered)
+  // unit construction and repairs
+  HistEvent *he = static_cast<HistEvent *>( events.Head() );
+  while ( he ) {
+    Unit *u, *u2;
+
+    if ( he->type == History::HIST_MOVE ) {
+      u = mission->GetUnit( he->data[0] );
+      if ( u ) {
+        RemoveUnit( u );
+
+        do {
+          MoveUnit( u, (Direction)he->data[1] );
+          he = static_cast<HistEvent *>( he->Next() );
+        } while ( he && he->type == History::HIST_MOVE && he->data[0] == u->ID() );
+
+        EndMovement( u );
+        CheckEvents();
+        continue;
+      }
+
+    } else if ( he->type == History::HIST_UNIT ) {
+
+      switch ( he->data[1] ) {
+      case History::HIST_UEVENT_CREATE:
+        u = history.GetDummy( he->data[0] );
+        u2 = mission->GetUnit( he->data[0] );
+
+        // units can be created by players (in factories) or by events.
+        // we only want to execute those triggered by players, since we'd
+        // otherwise create units twice. a cleaner solution would be
+        // to clearly indicate for each event who/what caused it, but this
+        // little hack does its job as well: don't build the unit if
+        // another unit with the same ID already exists
+        if ( u && !u2 ) {
+          mission->CreateUnit( u->Type()->ID(), *u->Owner(), u->Position(),
+                               (Direction)u->Facing(), u->GroupSize(), u->XP() );
+          CheckEvents();
+        }
+        break;
+      case History::HIST_UEVENT_REPAIR:
+        u = mission->GetUnit( he->data[0] );
+        if ( u ) {
+          UnitContainer *uc = dynamic_cast<UnitContainer *>
+                              (mission->GetMap().GetMapObject( u->Position() ));
+          if ( uc )
+            uc->SetCrystals( uc->Crystals() - CRYSTALS_REPAIR );
+
+          if ( mission->GetHistory() )
+            mission->GetHistory()->RecordUnitEvent( *u, History::HIST_UEVENT_REPAIR );
+
+          u->Repair();
+        }
+        break;
+      default:
+        // ignore
+        break;
+      }
+
+    } else if ( he->type == History::HIST_ATTACK ) {
+      u = mission->GetUnit( he->data[0] );
+      u2 = mission->GetMap().GetUnit( Point( he->data[1], he->data[2] ) );
+
+      if ( u && u2 )
+        mission->RegisterBattle( u, u2 );
+
+    } else if ( he->type == History::HIST_COMBAT ) {
+      u = mission->GetUnit( he->data[0] );
+      u2 = mission->GetUnit( he->data[1] );
+
+      if ( u && u2 ) {
+        Combat cmb( u, u2 );
+        Point casualties( he->data[3], he->data[2] );
+        ResolveBattle( &cmb, &casualties );
+        CheckEvents();
+      }
+
+    } else if ( he->type == History::HIST_TRANSPORT_CRYSTALS ) {
+      u = mission->GetUnit( he->data[0] );
+
+      if ( u ) {
+        UnitContainer *uc = dynamic_cast<UnitContainer *>
+                            (mission->GetMap().GetMapObject( u->Position() ));
+        if ( uc ) {
+          Transport *t = static_cast<Transport*>( u );
+          uc->SetCrystals( uc->Crystals() - he->data[1] );
+          u2->SetFlags( U_MOVED|U_DONE );
+          t->SetCrystals( t->Crystals() + he->data[1] );
+        }
+      }
+
+    } else if ( he->type == History::HIST_TRANSPORT_UNIT ) {
+      u = mission->GetUnit( he->data[0] );
+      u2 = mission->GetUnit( he->data[1] );
+
+      if ( u && u2 ) {
+        UnitContainer *uc = dynamic_cast<UnitContainer *>
+                            (mission->GetMap().GetMapObject( u->Position() ));
+        if ( uc ) {
+          Transport *t = static_cast<Transport*>( u );
+          uc->RemoveUnit( u2 );
+          t->InsertUnit( u2 );
+        }
+      }
+    }
+
+    he = static_cast<HistEvent *>( he->Next() );
+  }
+
+  // make sure the battles are not rerun at EndTurn()
+  mission->GetBattles().Clear();
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Game::GameMenu
 // DESCRIPTION: Pop up a MenuWindow with general game options like
@@ -1558,10 +1860,14 @@ void Game::GameMenu( void ) {
 #ifndef DISABLE_SOUND
   menu->AddItem( 1, G_BUTTON_SOUND_OPTIONS, 0, MSG(MSG_B_OPT_AUDIO) );
 #endif
+  menu->AddItem( 1, G_BUTTON_KEYBOARD_OPTIONS, 0, MSG(MSG_B_OPT_KEYBOARD) );
   menu->AddBar( 0 );
-  menu->AddItem( 0, G_BUTTON_SAVE, 
-                 (mission->GetFlags() & GI_PBEM) ? WIDGET_DISABLED : 0,
-                 MSG(MSG_B_SAVE_GAME) );
+  menu->AddItem( 0, G_BUTTON_SAVE,
+        (mission->GetFlags() & GI_PBEM || CFOptions.GetGameType() == GTYPE_NET_CLIENT) ?
+        WIDGET_DISABLED : 0, MSG(MSG_B_SAVE_GAME) );
+
+  if (view->IsFullScreen())
+    menu->AddItem( 0, G_BUTTON_MINIMIZE_WINDOW, 0, MSG(MSG_OPT_KEY_MINIMIZE) );
   menu->AddItem( 0, G_BUTTON_ABORT,    0, MSG(MSG_B_MAIN_MENU) );
   menu->AddItem( 0, G_BUTTON_QUIT,     0, MSG(MSG_B_QUIT) );
 
@@ -1658,6 +1964,11 @@ GUI_Status Game::WidgetActivated( Widget *button, Window *win ) {
     view->CloseWindow( win );
     new LocaleOptionsWindow( this, view );
     break;
+  case G_BUTTON_KEYBOARD_OPTIONS:
+    static_cast<MenuWindow *>(win)->CloseParent();
+    view->CloseWindow( win );
+    new KeyboardOptionsWindow( view );
+    break;
   case G_BUTTON_VIDEO_OPTIONS:
     static_cast<MenuWindow *>(win)->CloseParent();
     view->CloseWindow( win );
@@ -1713,6 +2024,10 @@ GUI_Status Game::WidgetActivated( Widget *button, Window *win ) {
     view->CloseWindow( win );
     Undo();
     break;
+  case G_BUTTON_MINIMIZE_WINDOW:
+    view->CloseWindow( win );
+    SDL_WM_IconifyWindow();
+    break;
   }
 
   return rc;
