@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ extern Options CFOptions;
 // RETURNS    : -1 on error, 0 otherwise
 ////////////////////////////////////////////////////////////////////////
 
-int HistEvent::Load( File &file ) {
+int HistEvent::Load( MemBuffer &file ) {
   type = file.Read8();
   for ( int i = 0; i < 4; ++i ) data[i] = file.Read16();
   return 0;
@@ -47,7 +47,7 @@ int HistEvent::Load( File &file ) {
 // RETURNS    : 0 on success, non-0 otherwise
 ////////////////////////////////////////////////////////////////////////
 
-int HistEvent::Save( File &file ) const {
+int HistEvent::Save( MemBuffer &file ) const {
   file.Write8( type );
   for ( int i = 0; i < 4; ++i ) file.Write16( data[i] );
   return 0;
@@ -62,7 +62,7 @@ int HistEvent::Save( File &file ) const {
 // RETURNS    : -1 on error, number of events loaded otherwise
 ////////////////////////////////////////////////////////////////////////
 
-short History::Load( File &file, Mission &mission ) {
+short History::Load( MemBuffer &file, Mission &mission ) {
   unsigned short num_events = file.Read16();
 
   if ( num_events > 0 ) {
@@ -86,12 +86,33 @@ short History::Load( File &file, Mission &mission ) {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : History::Save
 // DESCRIPTION: Save a history to file.
-// PARAMETERS : file - open file descriptor
+// PARAMETERS : file    - open file descriptor
+//              network - do some special processing if saving for
+//                        sending over the wire in a network game
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int History::Save( File &file ) const {
-  unsigned short num = events.CountNodes();
+int History::Save( MemBuffer &file, bool network /* = false */ ) const {
+  unsigned short num;
+
+  if ( network ) {
+    num = 0;
+    for ( HistEvent *he = static_cast<HistEvent *>( events.Head() );
+          he; he = static_cast<HistEvent *>( he->Next() ) ) {
+      // don't resend any events
+      if ( !he->processed ) {
+        if ( he->type == HIST_MOVE ||
+             (he->type == HIST_UNIT && he->data[1] != HIST_UEVENT_DESTROY) ||
+             he->type == HIST_ATTACK || he->type == HIST_COMBAT ||
+             he->type == HIST_TRANSPORT_CRYSTALS || he->type == HIST_TRANSPORT_UNIT )
+          ++num;
+        else
+          he->processed = true;
+      }
+
+    }
+  } else num = events.CountNodes();
+
   file.Write16( num );
   if ( num == 0 ) return 0;
 
@@ -100,7 +121,8 @@ int History::Save( File &file ) const {
   // save events
   for ( HistEvent *he = static_cast<HistEvent *>( events.Head() );
         he; he = static_cast<HistEvent *>( he->Next() ) )
-    he->Save( file );
+    if ( !network || !he->processed )
+      he->Save( file );
 
   // save units
   for ( Unit *u = static_cast<Unit *>( units.Head() );
@@ -170,7 +192,7 @@ void History::Replay( MapWindow *mapwin ) {
       progwin = new ProgressWindow( mapwin->Width() / 4,
               mapwin->Height() - view->SmallFont()->Height() - 30,
               mapwin->Width() / 2, view->SmallFont()->Height() + 20,
-              0, count, WIN_PROG_ABORT, view );
+              0, count, NULL, WIN_PROG_ABORT|WIN_PROG_DEFAULT, view );
     }
 
     while ( he && !abort ) {
@@ -194,7 +216,7 @@ void History::Replay( MapWindow *mapwin ) {
 
       if ( !quick ) {
         progwin->Advance( 1 );
-        abort = progwin->Aborted();
+        abort = progwin->Cancelled();
       }
 
       he = static_cast<HistEvent *>( he->Next() );
@@ -335,8 +357,8 @@ int History::RecordCombatEvent( const Combat &combat, unsigned char loss1,
   HistEvent *he = new HistEvent;
 
   he->type = HIST_COMBAT;
-  he->data[0] = combat.c_att->ID();
-  he->data[1] = combat.c_def->ID();
+  he->data[0] = combat.GetAttacker()->ID();
+  he->data[1] = combat.GetDefender()->ID();
   he->data[2] = loss1;
   he->data[3] = loss2;
 
@@ -408,23 +430,72 @@ int History::RecordMsgEvent( short title, unsigned short msg, short pos ) {
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : History::RecordUnitEvent
-// DESCRIPTION: Record the creation of a new unit.
-// PARAMETERS : unit - newly created unit
+// DESCRIPTION: Record the creation or destruction of a unit.
+// PARAMETERS : unit - created or destroyed unit
+//              type - type of event (created, destroyed, repaired)
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int History::RecordUnitEvent( const Unit &unit ) {
+int History::RecordUnitEvent( const Unit &unit, HistUnitEventType type ) {
   HistEvent *he = new HistEvent;
 
   he->type = HIST_UNIT;
   he->data[0] = unit.ID();
-  he->data[1] = he->data[2] = he->data[3] = -1;
+  he->data[1] = type;
+  he->data[2] = he->data[3] = -1;
 
   events.AddTail( he );
 
-  Unit *clone = new Unit( unit );
-  clone->SetFlags( U_BUSY|U_DUMMY );
-  units.AddTail( clone );
+  if ( type == HIST_UEVENT_CREATE ) {
+    Unit *clone = new Unit( unit );
+    clone->SetFlags( U_BUSY|U_DUMMY );
+    units.AddTail( clone );
+  }
+
+  return 0;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : History::RecordTransportEvent
+// DESCRIPTION: Record the (un)loading of crystals.
+// PARAMETERS : source   - outer container to give/receive crystals
+//              dest     - inner container to give/receive crystals
+//              crystals - number of crystals to move from source to
+//                         destination (may be negative)
+// RETURNS    : 0 on success, -1 on error
+////////////////////////////////////////////////////////////////////////
+
+int History::RecordTransportEvent( const UnitContainer &source,
+                            const Transport &dest, short crystals ) {
+  HistEvent *he = new HistEvent;
+
+  he->type = HIST_TRANSPORT_CRYSTALS;
+  he->data[0] = dest.ID();
+  he->data[1] = crystals;
+  he->data[2] = he->data[3] = -1;
+  events.AddTail( he );
+  return 0;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : History::RecordTransportEvent
+// DESCRIPTION: Record the movement of a unit from one container into
+//              another (without leaving the current hex).
+// PARAMETERS : source - outer container to remove unit from
+//              dest   - inner container to add unit to
+//              u      - unit to move from source to dest
+// RETURNS    : 0 on success, -1 on error
+////////////////////////////////////////////////////////////////////////
+
+int History::RecordTransportEvent( const UnitContainer &source,
+                            const Transport &dest, const Unit &u ) {
+  HistEvent *he = new HistEvent;
+
+  he->type = HIST_TRANSPORT_UNIT;
+  he->data[0] = dest.ID();
+  he->data[1] = u.ID();
+  he->data[2] = he->data[3] = -1;
+  events.AddTail( he );
   return 0;
 }
 
@@ -578,10 +649,8 @@ void History::ReplayCombatEvent( const HistEvent &event, MapWindow *mapwin ) {
 
   Combat cmb( Gam->GetMission()->GetUnit( event.data[0] ),
               Gam->GetMission()->GetUnit( event.data[1] ) );
-  cmb.aamod = (signed char)event.data[3];
-  cmb.ddmod = (signed char)event.data[2];
-
-  Gam->ResolveBattle( &cmb );
+  Point casualties( event.data[3], event.data[2] );
+  Gam->ResolveBattle( &cmb, &casualties );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -597,9 +666,9 @@ void History::ReplayUnitEvent( const HistEvent &event, MapWindow *mapwin ) {
   MapView *mv = mapwin->GetMapView();
   Map *map = mv->GetMap();
 
-  for ( Unit *u = static_cast<Unit *>( units.Head() );
-        u; u = static_cast<Unit *>( u->Next() ) ) {
-    if ( u->ID() == event.data[0] ) {
+  Unit *u = GetDummy( event.data[0] );
+  if ( u ) {
+    if ( event.data[1] == HIST_UEVENT_CREATE ) { // create unit
       u->Remove();
 
       if ( !u->IsSheltered() ) {
@@ -612,25 +681,38 @@ void History::ReplayUnitEvent( const HistEvent &event, MapWindow *mapwin ) {
         map->SetUnit( u, u->Position() );
       }
       mission->GetUnits().AddTail( u );
-      break;
+    } else if ( event.data[1] == HIST_UEVENT_DESTROY ) { // destroy unit
+      if ( !u->IsSheltered() ) {
+        map->SetUnit( NULL, u->Position() );
+        if ( mv->Enabled() ) {
+          mapwin->DisplayHex( u->Position() );
+          SDL_Delay( delay );
+          mapwin->FadeOutUnit( u->Image(), u->Position() );
+          SDL_Delay( delay );
+        }
+      }
     }
   }
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : History::EraseMoveEvents
-// DESCRIPTION: Erase all recorded movement for a spcific unit.
+// NAME       : History::UndoMove
+// DESCRIPTION: Erase all recorded movement and related events for a
+//              specific unit.
 // PARAMETERS : u - unit to erase movement for
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void History::EraseMoveEvents( const Unit &u ) {
+void History::UndoMove( const Unit &u ) {
   HistEvent *he = static_cast<HistEvent *>( events.Head() ), *tmp;
 
   while ( he ) {
     tmp = static_cast<HistEvent *>( he->Next() );
 
-    if ( (he->type == HIST_MOVE) && (he->data[0] == u.ID()) ) {
+    if ( ((he->type == HIST_MOVE) ||
+          (he->type == HIST_TRANSPORT_CRYSTALS) ||
+          (he->type == HIST_TRANSPORT_UNIT)) &&
+         (he->data[0] == u.ID()) ) {
       he->Remove();
       delete he;
     }
@@ -639,3 +721,30 @@ void History::EraseMoveEvents( const Unit &u ) {
   }
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : History::GetDummy
+// DESCRIPTION: Retrieve an internal unit definition.
+// PARAMETERS : id - unit identifier
+// RETURNS    : dummy for the requested unit, NULL if not found
+////////////////////////////////////////////////////////////////////////
+
+Unit *History::GetDummy( unsigned short id ) const {
+  Unit *u;
+  for ( u = static_cast<Unit *>( units.Head() );
+        u && u->ID() != id; u = static_cast<Unit *>( u->Next() ) );
+  return u;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : History::SetEventsProcessed
+// DESCRIPTION: Set all events to processed.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void History::SetEventsProcessed( void ) const {
+  for ( HistEvent *he = static_cast<HistEvent *>( events.Head() );
+        he; he = static_cast<HistEvent *>( he->Next() ) ) {
+    he->processed = true;
+  }
+}
