@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,23 +26,15 @@
 
 #include "combat.h"
 
-enum {  HIST_MOVE = 0,
-        HIST_ATTACK,
-        HIST_COMBAT,
-        HIST_TILE,      // map tile change
-        HIST_TILE_INTERNAL,
-        HIST_MSG,
-        HIST_UNIT
-};
-
 class HistEvent : public Node {
 public:
-  HistEvent( void ) {}
-  int Load ( File &file );
-  int Save( File &file ) const;
+  HistEvent( void ) : processed(false) {}
+  int Load ( MemBuffer &file );
+  int Save( MemBuffer &file ) const;
 
   unsigned char type;
   short data[4];
+  bool processed;
 };
 
 
@@ -50,9 +42,27 @@ class MapWindow;
 
 class History {
 public:
+  enum HistEventType {
+    HIST_MOVE = 0,
+    HIST_ATTACK,
+    HIST_COMBAT,
+    HIST_TILE,      // map tile change
+    HIST_TILE_INTERNAL,
+    HIST_MSG,
+    HIST_UNIT,
+    HIST_TRANSPORT_CRYSTALS,
+    HIST_TRANSPORT_UNIT
+  };
+
+  enum HistUnitEventType {
+    HIST_UEVENT_CREATE,
+    HIST_UEVENT_DESTROY,
+    HIST_UEVENT_REPAIR
+  };
+
   History( void ) {}
-  short Load( File &file, Mission &mission );
-  int Save( File &file ) const;
+  short Load( MemBuffer &file, Mission &mission );
+  int Save( MemBuffer &file, bool network = false ) const;
 
   void StartRecording( List &list );
 
@@ -63,11 +73,18 @@ public:
   int RecordMsgEvent( short title, unsigned short msg, short pos );
   int RecordTileEvent( unsigned short tile, unsigned short old,
                        short dx, short dy );
-  int RecordUnitEvent( const Unit &u );
+  int RecordTransportEvent( const UnitContainer &source,
+                            const Transport &dest, short crystals );
+  int RecordTransportEvent( const UnitContainer &source,
+                            const Transport &dest, const Unit &u );
+  int RecordUnitEvent( const Unit &u, HistUnitEventType type );
 
   void Replay( MapWindow *mapwin );
 
-  void EraseMoveEvents( const Unit &u );
+  void UndoMove( const Unit &u );
+  void SetEventsProcessed( void ) const;
+  const List &GetEvents( void ) const { return events; }
+  Unit *GetDummy( unsigned short id ) const;
 
 private:
   void BeginReplay( List &backup, Map *map );
@@ -89,5 +106,5 @@ private:
   Point lastpos;
 };
 
-#endif	/* _INCLUDE_HISTORY_H */
+#endif  /* _INCLUDE_HISTORY_H */
 
