@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,16 +31,19 @@
 #include "options.h"
 #include "msgs.h"
 
-#define IW_WIDGET_TYPE		0
-#define IW_WIDGET_MAPTYPE	1
-#define IW_WIDGET_LEVELS	2
-#define IW_WIDGET_START		3
-#define IW_WIDGET_OPTIONS	4
-#define IW_WIDGET_GENERAL_OPTIONS  5
-#define IW_WIDGET_LANGUAGE_OPTIONS 6
-#define IW_WIDGET_VIDEO_OPTIONS    7
-#define IW_WIDGET_SOUND_OPTIONS    8
-#define IW_WIDGET_DIFFICULTY	9
+enum {
+  IW_WIDGET_TYPE = 0,
+  IW_WIDGET_MAPTYPE,
+  IW_WIDGET_LEVELS,
+  IW_WIDGET_START,
+  IW_WIDGET_OPTIONS,
+  IW_WIDGET_GENERAL_OPTIONS,
+  IW_WIDGET_LANGUAGE_OPTIONS,
+  IW_WIDGET_VIDEO_OPTIONS,
+  IW_WIDGET_SOUND_OPTIONS,
+  IW_WIDGET_KEYBOARD_OPTIONS,
+  IW_WIDGET_DIFFICULTY
+};
 
 #define MAPTYPE_SINGLE   0
 #define MAPTYPE_CAMPAIGN 1
@@ -48,8 +51,6 @@
 
 extern Options CFOptions;
 
-const short GenericOptionsWindow::B_ID_OK = 1;
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : InitWindow::InitWindow
 // DESCRIPTION: Create the main menu window. The player(s) can select
@@ -139,16 +140,20 @@ void InitWindow::Rebuild( void ) {
   // create widgets
   Widget *wd;
 
-  const char *typelabels[4];
-  typelabels[0] = MSG(MSG_GAME_HOT_SEAT);
-  typelabels[1] = MSG(MSG_GAME_AI);
-  typelabels[2] = MSG(MSG_GAME_PBEM);
-  typelabels[3] = 0;
+  const char *typelabels[GTYPE_COUNT + 1];
+  typelabels[GTYPE_HOTSEAT] = MSG(MSG_GAME_HOT_SEAT);
+  typelabels[GTYPE_AI] = MSG(MSG_GAME_AI);
+  typelabels[GTYPE_PBEM] = MSG(MSG_GAME_PBEM);
+#ifndef DISABLE_NETWORK
+  typelabels[GTYPE_NET_SERVER] = MSG(MSG_GAME_NETWORK_SERVER);
+  typelabels[GTYPE_NET_CLIENT] = MSG(MSG_GAME_NETWORK_CLIENT);
+#endif
+  typelabels[GTYPE_COUNT] = 0;
 
   const char *mapslabels[4];
-  mapslabels[0] = MSG(MSG_MAPS_SINGLES);
-  mapslabels[1] = MSG(MSG_MAPS_CAMPAIGNS);
-  mapslabels[2] = MSG(MSG_MAPS_SAVES);
+  mapslabels[MAPTYPE_SINGLE] = MSG(MSG_MAPS_SINGLES);
+  mapslabels[MAPTYPE_CAMPAIGN] = MSG(MSG_MAPS_CAMPAIGNS);
+  mapslabels[MAPTYPE_SAVE] = MSG(MSG_MAPS_SAVES);
   mapslabels[3] = 0;
 
   const char *difflabels[4];
@@ -179,15 +184,16 @@ void InitWindow::Rebuild( void ) {
            0, hcaplbl, 0, difflabels, this );
 
   levwidget = new TextListWidget( IW_WIDGET_LEVELS,
-                    5, mtypewidget->TopEdge() + wdh + 5,
-                    w/2 - 7, h - (wdh + 5) * 3 - 5, &levels,
-                    -1, WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY,
-                    NULL, this );
+      5, mtypewidget->TopEdge() + wdh + 5,
+      w/2 - 7, h - (wdh + 5) * 3 - 5, &levels, -1,
+      WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY|
+      (CFOptions.GetGameType() == GTYPE_NET_CLIENT ? WIDGET_DISABLED : 0),
+      NULL, this );
   levwidget->SetHook( this );
 
   maxmap = Rect( w/2 + 2, levwidget->TopEdge(), w/2 - 7, levwidget->Height() );
 
-  // of the these two widgets, only one is actually present at a time
+  // of these two widgets, only one is actually present at a time
   mapwidget = new MapWidget( 0, maxmap.x, maxmap.y, maxmap.w, maxmap.h,
               0, this );
   campinfowidget = new TextScrollWidget( 0, maxmap.x, maxmap.y,
@@ -259,57 +265,20 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
       }
     }
     break;
-  case IW_WIDGET_START:
-    n = static_cast<TLWNode *>( levwidget->Selected() );
-    if ( !n ) new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_NO_MAP), WIN_CLOSE_ESC, view );
-    else {
-      string *lname = (string *)n->UserData();
+  case IW_WIDGET_START: {
+    const char *name = NULL;
+    bool err = false;
 
-      Gam = new Game( view );
-      int val = Gam->Load( lname->c_str() );
-      if ( val  ) {
-        NoteWindow *nw = new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_LOAD_MAP), 0, view );
-        nw->SetButtonID( 0, GUI_RESTART );
-      } else {
-        Mission *m = Gam->GetMission();
-        Audio::StopMusic( CF_MUSIC_FADE_TIME );
-        view->CloseWindow( this );
-
-        switch (mtypewidget->GetValue() ) {
-        case MAPTYPE_SINGLE:
-          if ( CFOptions.IsAI() ) {
-            string pnames( "[_1] " );
-            pnames += m->GetPlayer(PLAYER_ONE).Name();
-            pnames += "|[_2] ";
-            pnames += m->GetPlayer(PLAYER_TWO).Name();
-
-            DialogWindow *pwin = new DialogWindow( MSG(MSG_PLAYER_SELECTION),
-                  MSG(MSG_ASK_SIDE), pnames, 0, 0, view );
-            pwin->SetButtonID( 0, PLAYER_ONE );
-            pwin->SetButtonID( 1, PLAYER_TWO );
-            short rc = pwin->EventLoop();
-            view->CloseWindow( pwin );
-            m->GetPlayer( rc ).SetType( HUMAN );
-          } else {
-            m->GetPlayer(PLAYER_ONE).SetType( HUMAN );
-            m->GetPlayer(PLAYER_TWO).SetType( HUMAN );
-          }
-          break;
-        case MAPTYPE_CAMPAIGN:
-          m->GetPlayer(PLAYER_ONE).SetType( HUMAN );
-          break;
-        }
-
-        if ( mtypewidget->GetValue() != MAPTYPE_SAVE )
-          m->SetHandicap( 1 << diffwidget->GetValue() );
-
-        view->Flood( Color(CF_COLOR_SHADOW) );
-        view->Update();
-        Gam->InitWindows();
-        rc = Gam->StartTurn();         // let the games begin...
-      }
+    if ( !CFOptions.IsNetwork() || (CFOptions.GetGameType() != GTYPE_NET_CLIENT) ) {
+      n = static_cast<TLWNode *>( levwidget->Selected() );
+      if ( !n ) {
+        new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_NO_MAP), WIN_CLOSE_ESC, view );
+        err = true;
+      } else name = ((string *)n->UserData())->c_str();
     }
-    break;
+
+    if ( !err ) rc = StartGame( name );
+    break; }
   case IW_WIDGET_MAPTYPE:
     switch ( mtypewidget->GetValue() ) {
     case MAPTYPE_SINGLE:
@@ -354,6 +323,7 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
 #ifndef DISABLE_SOUND
     menu->AddItem( 0, IW_WIDGET_SOUND_OPTIONS, 0, MSG(MSG_B_OPT_AUDIO) );
 #endif
+    menu->AddItem( 0, IW_WIDGET_KEYBOARD_OPTIONS, 0, MSG(MSG_B_OPT_KEYBOARD) );
     menu->SetPosition(
       Point( x + button->LeftEdge() + 1,
              y + button->TopEdge() + button->Height() - 1 )
@@ -363,7 +333,18 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
     break; }
 
   case IW_WIDGET_TYPE:
-    CFOptions.SetGameType( gtypewidget->GetValue() );
+    CFOptions.SetGameType( (GameType)gtypewidget->GetValue() );
+    if ( CFOptions.GetGameType() == GTYPE_NET_CLIENT ) {
+      diffwidget->Disable();
+      mtypewidget->Disable();
+      levwidget->Disable();
+    } else {
+      diffwidget->Enable();
+      mtypewidget->Enable();
+      levwidget->Enable();
+    }
+    Draw();
+    Show();
     break;
   case IW_WIDGET_GENERAL_OPTIONS:
     view->CloseWindow( win );
@@ -377,6 +358,10 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
     view->CloseWindow( win );
     new LocaleOptionsWindow( 0, view );
     break;
+  case IW_WIDGET_KEYBOARD_OPTIONS:
+    view->CloseWindow( win );
+    new KeyboardOptionsWindow( view );
+    break;
 #ifndef DISABLE_SOUND
   case IW_WIDGET_SOUND_OPTIONS:
     view->CloseWindow( win );
@@ -388,6 +373,171 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
   return rc;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : InitWindow::StartGame
+// DESCRIPTION: Start a new (or old) game.
+// PARAMETERS : filename - name of file to load game from
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status InitWindow::StartGame( const char *filename ) {
+  Game *game = NULL;
+  Mission *m;
+
+  if ( filename ) {
+    game = new Game( view );
+
+    if ( !game->Load( filename ) ) {
+      m = game->GetMission();
+
+      // apply handicap setting
+      if ( !(m->GetFlags() & GI_SAVEFILE) )
+        m->SetHandicap( 1 << diffwidget->GetValue() );
+    } else {
+      delete game;
+      NoteWindow *nw = new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_LOAD_MAP), 0, view );
+      nw->SetButtonID( 0, GUI_RESTART );
+      return GUI_OK;
+    }
+  }
+
+#ifndef DISABLE_NETWORK
+  if ( CFOptions.IsNetwork() || (game && m->GetFlags() & GI_NETWORK) ) {
+    int err;
+    TCPConnection *connection = new TCPConnection();
+
+    // ask for network parameters (server/port)
+    bool is_server = (game != NULL);
+    short pid;
+
+    if ( is_server )
+      // ask the server which side he'd like to play
+      pid = AskForSide( *m );
+
+    NetworkSetupWindow *nsw =
+      new NetworkSetupWindow( is_server, view );
+
+    do {
+      err = 0;
+      GUI_Status rc = nsw->EventLoop();
+      if ( rc <= GUI_OK ) {
+        delete game;
+        delete connection;
+        view->CloseWindow( nsw );
+        return GUI_OK;
+      }
+
+      nsw->SetConnecting( true );
+
+      // if server, send initial data to client
+      if ( is_server ) {
+        DynBuffer send;
+        // put player ID for client at start of sync buffer
+        send.Write8( pid^1 );
+
+        err  = connection->Open( NULL, nsw->GetPort(), nsw );
+
+        if ( err || game->Save( send ) ||
+             !connection->Send( send ) ) {
+          nsw->SetConnecting( false );
+          connection->Close();
+          if ( !err ) err = 1;
+        } else {
+          // make sure we won't send any events twice
+          if ( m->GetHistory() )
+            m->GetHistory()->SetEventsProcessed();
+        }
+      }
+
+      // if client, wait for game data to arrive
+      else {
+        DynBuffer *recv = NULL;
+
+        err = connection->Open( nsw->GetServer(), nsw->GetPort(), nsw );
+
+        if ( !err ) {
+          recv = connection->Receive( nsw );
+
+          if ( recv ) {
+            game = new Game( view );
+            // our player ID is sent first
+            pid = recv->Read8();
+            if ( game->Load( *recv ) != 0 )  {
+              err = -1;
+            }
+            delete recv;
+          } else err = -1;
+        }
+
+        if ( err ) {
+          nsw->SetConnecting( false );
+          connection->Close();
+          delete game;
+          game = NULL;
+        }
+      }
+
+      if ( err == -1 ) {
+        NoteWindow *nw = new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_LOAD_MAP), 0, view );
+        nw->EventLoop();
+        view->CloseWindow( nw );
+      }
+    }  while ( err );
+
+    m = game->GetMission();
+    m->GetPlayer( pid ).SetRemote( false );
+    m->GetPlayer( pid^1 ).SetRemote( true );
+    game->SetNetworkConnection( connection );
+
+    view->CloseWindow( nsw );
+  }
+#endif // !DISABLE_NETWORK
+
+  Audio::StopMusic( CF_MUSIC_FADE_TIME );
+  view->CloseWindow( this );
+
+  if ( mtypewidget->GetValue() == MAPTYPE_SINGLE ) {
+    if ( m->GetFlags() & GI_AI ) {
+      short pid = AskForSide( *m );
+      m->GetPlayer( pid ).SetType( HUMAN );
+    } else {
+      m->GetPlayer(PLAYER_ONE).SetType( HUMAN );
+      m->GetPlayer(PLAYER_TWO).SetType( HUMAN );
+    }
+  } else if ( mtypewidget->GetValue() == MAPTYPE_CAMPAIGN ) {
+    m->GetPlayer(PLAYER_ONE).SetType( HUMAN );
+  }
+
+  view->Flood( Color(CF_COLOR_SHADOW) );
+  view->Update();
+  Gam = game;
+  Gam->InitWindows();
+  return Gam->StartTurn();         // let the games begin...
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : InitWindow::AskForSide
+// DESCRIPTION: Ask the user which side he wants to play on.
+// PARAMETERS : m - mission to chose side for
+// RETURNS    : identifier of the player/side chosen
+////////////////////////////////////////////////////////////////////////
+
+short InitWindow::AskForSide( Mission &m ) const {
+  string pnames( "[_1] " );
+  pnames += m.GetPlayer(PLAYER_ONE).Name();
+  pnames += "|[_2] ";
+  pnames += m.GetPlayer(PLAYER_TWO).Name();
+
+  DialogWindow *pwin = new DialogWindow( MSG(MSG_PLAYER_SELECTION),
+                       MSG(MSG_ASK_SIDE), pnames, 0, 0, view );
+  pwin->SetButtonID( 0, PLAYER_ONE );
+  pwin->SetButtonID( 1, PLAYER_TWO );
+  short rc = pwin->EventLoop();
+  view->CloseWindow( pwin );
+
+  return rc;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : InitWindow::CompleteFilesList
 // DESCRIPTION: For the levels list display we need some information
@@ -448,6 +598,7 @@ void InitWindow::CompleteFilesList( TLWList &list ) {
         title += ' ';
         title += '(';
         if ( m->GetFlags() & GI_PBEM ) title += MSG(MSG_TAG_PBEM);
+        else if ( m->GetFlags() & GI_NETWORK ) title += MSG(MSG_TAG_NET);
         else if ( m->GetFlags() & GI_SAVEFILE ) {
           if ( !m->GetPlayer(PLAYER_ONE).IsHuman() ||
                !m->GetPlayer(PLAYER_TWO).IsHuman() )
@@ -518,14 +669,33 @@ Mission *InitWindow::LoadMission( const char *filename, bool full ) const {
 ////////////////////////////////////////////////////////////////////////
 
 TitleWindow::TitleWindow( View *view ) : Window( WIN_CENTER, view ) {
-  string tname = get_data_dir();
-  append_path_delim( tname );
-  tname.append( CF_TITLE_SCREEN );
+  bool error = false;
 
-  if ( LoadBMP( tname.c_str() ) ) SetSize( 0, 0, 0, 0 );
+  string tname, tbase = get_data_dir();
+  append_path_delim( tbase );
+
+  // try to load resolution-optimized splash screen first
+  tbase.append( "title" );
+  tname = tbase + StringUtil::tostring( view->Width() ) + ".bmp";
+  error = (LoadBMP( tname.c_str() ) != 0);
+  if ( error ) {
+    if ( view->Width() < 320 ) {
+      // one more attempt
+      tname = tbase + "320.bmp";
+      error = (LoadBMP( tname.c_str() ) != 0);
+    }
+
+    if ( error ) {
+      // use the standard splash screen
+      tname = tbase + ".bmp";
+      error = (LoadBMP( tname.c_str() ) != 0);
+    }
+  }
+
+  if ( error ) SetSize( 0, 0, 0, 0 );
   else {
     DisplayFormat();
-    Center( *view ); 
+    Center( *view );
 
     Show();
   }
@@ -556,10 +726,17 @@ GUI_Status TitleWindow::HandleEvent( const SDL_Event &event ) {
 
 VideoOptionsWindow::VideoOptionsWindow( View *view ) :
      GenericOptionsWindow( MSG(MSG_OPTIONS_VIDEO), view ) {
+#ifdef _WIN32_WCE
+  SDL_Rect std_sizes[] = { { 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN) },
+         { 0, 0, GetSystemMetrics(SM_CYSCREEN), GetSystemMetrics(SM_CXSCREEN) } };
+#else
   SDL_Rect std_sizes[] = { { 0, 0, 1280, 1024 }, { 0, 0, 1024, 768 },
                            { 0, 0, 800, 600}, { 0, 0, 640, 480 } };
-  SDL_Rect *std_modes[5];
+#endif
+  const int n_modes = sizeof(std_sizes)/sizeof(SDL_Rect);
+  SDL_Rect *std_modes[n_modes + 1];
   short current = -1;
+  int i;
 
   // create modes list
   SDL_Rect **sdl_modes = SDL_ListModes( NULL, SDL_FULLSCREEN );
@@ -567,15 +744,13 @@ VideoOptionsWindow::VideoOptionsWindow( View *view ) :
   // if any mode is ok offer some standard ones
   if ( sdl_modes == (SDL_Rect **)-1 ) {
     sdl_modes = std_modes;
-    std_modes[0] = &std_sizes[0];
-    std_modes[1] = &std_sizes[1];
-    std_modes[2] = &std_sizes[2];
-    std_modes[3] = &std_sizes[3];
-    std_modes[4] = NULL;
+    for ( i = 0; i < n_modes; ++i )
+      std_modes[i] = &std_sizes[i];
+    std_modes[n_modes] = NULL;
   }
- 
+
   if ( sdl_modes != NULL ) {
-    for ( int i = 0; sdl_modes[i]; ++i ) AddMode( sdl_modes[i] );
+    for ( i = 0; sdl_modes[i]; ++i ) AddMode( sdl_modes[i] );
 
     // add current mode
     SDL_Rect screen = { 0, 0, view->Width(), view->Height() };
@@ -702,8 +877,8 @@ GUI_Status VideoOptionsWindow::WidgetActivated( Widget *button, Window *win ) {
 // NAME       : GenericOptionsWindow::SetLayout
 // DESCRIPTION: Set the size of the window. This must be called before
 //              trying to display it or using GetBounds().
-// PARAMETERS : w     - width (only the part needed by the subclass)
-//              h     - height (only the part needed by the subclass)
+// PARAMETERS : w - width (only the part needed by the subclass)
+//              h - height (only the part needed by the subclass)
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
@@ -980,6 +1155,186 @@ GUI_Status LocaleOptionsWindow::WidgetActivated( Widget *widget, Window *win ) {
   return rc;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : KeyboardOptionsWindow::KeyboardOptionsWindow
+// DESCRIPTION: Show the keyboard options menu.
+// PARAMETERS : view - view to attach the window to
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+KeyboardOptionsWindow::KeyboardOptionsWindow( View *view ) :
+     GenericOptionsWindow( MSG(MSG_OPTIONS_KEYBOARD), view ),
+     last( -1 ), request( 0 ) {
+  SetLayout( sfont->Width() * 20 + 20, 10 * (sfont->Height() + 2) + 25 );
+
+  // create widgets
+  const Rect &b = GetBounds();
+
+  RebuildKeyMap();
+
+  fncwidget = new TextListWidget( 0, b.x, b.y, b.w, b.h, &functions, -1,
+                  WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY,
+                  NULL, this);
+  fncwidget->SetKey( SDLK_SPACE );
+  fncwidget->SetHook( this );
+
+  Draw();
+  Show();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : KeyboardOptionsWindow::HandleEvent
+// DESCRIPTION: React to user input.
+// PARAMETERS : event - event received by the event handler
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status KeyboardOptionsWindow::HandleEvent( const SDL_Event &event ) {
+  GUI_Status rc;
+
+  if (request != NULL) {
+    if (event.type == SDL_KEYDOWN ) {
+      switch (event.key.keysym.sym) {
+      case SDLK_UNKNOWN:
+      // reserved for cursor movement
+      case SDLK_LEFT: case SDLK_RIGHT: case SDLK_UP: case SDLK_DOWN:
+      // reserved for map scrolling
+      case SDLK_KP1: case SDLK_KP2: case SDLK_KP3:
+      case SDLK_KP4: case SDLK_KP6: case SDLK_KP7:
+      case SDLK_KP8: case SDLK_KP9:
+        Audio::PlaySfx( Audio::SND_GUI_ERROR, 0 );
+        break;
+      default:
+        AssignKey( event.key.keysym.sym );
+      // fall through
+      case SDLK_ESCAPE:
+        view->CloseWindow( request );
+        request = NULL;
+        break;
+      }
+    }
+    rc = GUI_OK;
+  } else rc = GenericOptionsWindow::HandleEvent( event );
+  return rc;
+}
+
+/////////////////////////////////////////////////////////////////////////
+// NAME       : KeyboardOptionsWindow::WidgetActivated
+// DESCRIPTION: User pressed OK, leave window.
+// PARAMETERS : widget - calling widget
+//              win    - pointer to active window
+// RETURNS    : GUI_CLOSE
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status KeyboardOptionsWindow::WidgetActivated( Widget *widget, Window *win ) {
+  GUI_Status rc;
+
+  if (widget == fncwidget) {
+    TLWNode *n = static_cast<TLWNode *>(fncwidget->Selected());
+
+    if (n && (n->ID() == last))
+      request = new KeyboardPressKeyWindow( this, view );
+
+    last = n->ID();
+    rc = GUI_OK;
+  } else rc = GUI_CLOSE;
+
+  return rc;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : KeyboardOptionsWindow::AssignKey
+// DESCRIPTION: Assign a key to an action.
+// PARAMETERS : key - key to assign to currently selected action
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void KeyboardOptionsWindow::AssignKey( SDLKey key ) {
+  TLWNode *node = static_cast<TLWNode *>( fncwidget->Selected() );
+
+  if ( node != NULL ) {
+    KeyBinding bind = (KeyBinding)node->ID();
+    const SDLKey *keymap = CFOptions.GetKeyBindings();
+
+    switch (key) {
+    case SDLK_DELETE:
+    case SDLK_BACKSPACE:
+      CFOptions.SetKeyBinding( bind, SDLK_UNKNOWN );
+      break;
+    default:
+      for (int i = 0; i < KEYBIND_COUNT; ++i) {
+        if (keymap[i] == key)
+          CFOptions.SetKeyBinding( (KeyBinding)i, keymap[bind] ); // avoid dupes
+      }
+      CFOptions.SetKeyBinding( bind, key );
+      break;
+    }
+
+    Audio::PlaySfx( Audio::SND_GUI_PRESSED, 0 );
+
+    RebuildKeyMap();
+
+    fncwidget->DrawNodes();
+    fncwidget->Show();
+  }
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : KeyboardOptionsWindow::RebuildKeyMap
+// DESCRIPTION: Update the list of actions to include the currently
+//              assigned key commands.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void KeyboardOptionsWindow::RebuildKeyMap( void ) {
+  const SDLKey *keymap = CFOptions.GetKeyBindings();
+  string name;
+
+  functions.Clear();
+
+  for (int bind = 0; bind < KEYBIND_COUNT; ++bind) {
+    name = MSG(MSG_OPT_KEY_MINIMIZE + bind);
+
+    if (keymap[bind] != 0) {
+      name.append("  -  ");
+
+      if ((keymap[bind] >= 32) && (keymap[bind] < 255))
+        name.append( StringUtil::strprintf("'%c'", (char)keymap[bind]) );
+      else if ((keymap[bind] >= SDLK_F1) && (keymap[bind] <= SDLK_F15))
+        name.append( StringUtil::strprintf("F%d",((int)keymap[bind] - SDLK_F1)+1) );
+      else
+        name.append( StringUtil::strprintf("#%d",(int)keymap[bind]) );
+    }
+    functions.AddTail(new TLWNode(name.c_str(), NULL, bind));
+  }
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : KeyboardPressKeyWindow::KeyboardPressKeyWindow
+// DESCRIPTION: Ask the user to press a key. All events are handed down
+//              to the parent window.
+// PARAMETERS : parent - parent window
+//              view   - view to attach the window to
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+KeyboardPressKeyWindow::KeyboardPressKeyWindow( Window *parent, View *view ) :
+                        Window( WIN_CENTER, view ), parent(parent) {
+  const char *msg = MSG(MSG_PRESS_KEY);
+  unsigned short width = parent->Width() - 20;
+  unsigned short height = SmallFont()->TextHeight( msg, width - 20, 2 ) + 20;
+
+  SetSize( width, height );
+
+  new TextWidget( 0, 5, 5, w - 10, h - 10, msg, WIDGET_ALIGN_CENTER, NULL, this );
+
+  Draw();
+  Show();
+}
+
+#ifndef DISABLE_SOUND
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : SoundOptionsWindow::SoundOptionsWindow
 // DESCRIPTION: Show the sound options menu.
@@ -987,13 +1342,6 @@ GUI_Status LocaleOptionsWindow::WidgetActivated( Widget *widget, Window *win ) {
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-#ifndef DISABLE_SOUND
-
-const short SoundOptionsWindow::B_ID_SFX       = 10;
-const short SoundOptionsWindow::B_ID_MUSIC     = 11;
-const short SoundOptionsWindow::S_ID_VOL_SFX   = 12;
-const short SoundOptionsWindow::S_ID_VOL_MUSIC = 13;
-
 SoundOptionsWindow::SoundOptionsWindow( View *view ) :
      GenericOptionsWindow( MSG(MSG_OPTIONS_AUDIO), view ),
      volgfx( view->GetSystemIcons(), 145, 46, 12 ,12 ) {
@@ -1112,5 +1460,201 @@ void SoundOptionsWindow::Draw( void ) {
   volgfx.Draw( this, xoff,
          musicvol->TopEdge() + (musicvol->Height() - volgfx.Height())/2 );
 }
-#endif
+#endif  // !DISABLE_SOUND
 
+#ifndef DISABLE_NETWORK
+
+#define DEFAULT_PORT  9999
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NetworkSetupWindow::NetworkSetupWindow
+// DESCRIPTION: Ask user for network parameters. If we are hosting the
+//              game, ask for the local port to use, otherwise ask for
+//              server address/name and port.
+// PARAMETERS : server - whether we are the host, or the client
+//              view   - view to add window to
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+NetworkSetupWindow::NetworkSetupWindow( bool server, View *view ) :
+       Window( WIN_CENTER, view ),
+       server(server), connecting(false), address(NULL), port(NULL),
+       title(MSG(server ? MSG_NET_CONFIG_SERVER : MSG_NET_CONFIG_CLIENT)),
+       title_width(lfont->TextWidth(title)) {
+  const char *ip_label = MSG(MSG_B_SERVER);
+  const char *port_label = MSG(MSG_B_PORT);
+
+  unsigned short width, height, wdx, wdy, wdh;
+  wdy = lfont->Height() + 15;
+  wdh = sfont->Height() + 8;
+
+  wdx = sfont->TextWidth( port_label );
+  width = sfont->Width() * 5 + 25;
+  height = lfont->Height() + wdh * 2 + 30;
+
+  if (!server) {
+    unsigned short iplen = sfont->TextWidth( ip_label );
+    wdx = MAX( wdx, iplen );
+    width += sfont->Width() * 10;
+    height += wdh;
+  }
+
+  width += MAX( title_width, wdx );
+  width = MIN( width, view->Width() - 20 );
+
+  SetSize( width, height );
+
+  wdx += 10;
+
+  if (!server) {
+    address = new StringWidget( 0, wdx, wdy,
+              w - wdx - 15, wdh,
+              "localhost", 40, WIDGET_ALIGN_LEFT,
+              ip_label, this );
+    wdy += address->Height() + 5;
+  }
+
+  port = new NumberWidget( 0, wdx, wdy,
+         sfont->Width() * 5 + 20, wdh, DEFAULT_PORT, 1, 65536,
+         WIDGET_ALIGN_LEFT, port_label, this );
+
+  Widget *ok = new ButtonWidget( 0, 1, h - wdh - 1, (w - 2) / 2, wdh,
+               WIDGET_DEFAULT, MSG(MSG_B_OK), this );
+  ok->SetHook( this );
+
+  new ButtonWidget( GUI_CLOSE,
+                    ok->LeftEdge() + ok->Width() + 1, ok->TopEdge(),
+                    ok->Width(), wdh, 0, MSG(MSG_B_CANCEL), this );
+
+  Draw();
+  Show();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NetworkSetupWindow::~NetworkSetupWindow
+// DESCRIPTION: Destroy the window.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+NetworkSetupWindow::~NetworkSetupWindow( void ) {
+  if ( connecting ) {
+    delete address;
+    delete port;
+  }
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NetworkSetupWindow::SetConnecting
+// DESCRIPTION: Set current status - whether we are waiting for user
+//              input or connecting to remote peer.
+// PARAMETERS : connect - currently trying to connect to peer?
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void NetworkSetupWindow::SetConnecting( bool connect ) {
+  if (connect == connecting) return;
+
+  connecting = connect;
+
+  unsigned short wdh = sfont->Height() + 8;
+
+  if (connecting) {
+
+    if ( address ) RemoveWidget( address );
+    RemoveWidget( port );
+
+    RemoveAllWidgets();
+
+    new ButtonWidget( GUI_CLOSE,
+                    1, h - wdh - 1, w - 2, wdh,
+                    0, MSG(MSG_B_CANCEL), this );
+  } else {
+    RemoveAllWidgets();
+
+    Widget *ok = new ButtonWidget( 0, 1, h - wdh - 1, (w - 2) / 2, wdh,
+                 WIDGET_DEFAULT, MSG(MSG_B_OK), this );
+    ok->SetHook( this );
+
+    new ButtonWidget( GUI_CLOSE,
+                    ok->LeftEdge() + ok->Width() + 1, ok->TopEdge(),
+                    ok->Width(), ok->Height(), 0, MSG(MSG_B_CANCEL), this );
+
+    if ( address ) AddWidget( address );
+    AddWidget( port );
+  }
+
+  Draw();
+  Show();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NetworkSetupWindow::Draw
+// DESCRIPTION: Draw the network configuration window.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void NetworkSetupWindow::Draw( void ) {
+  Window::Draw();
+
+  const Rect box( 5, lfont->Height() + 10, w - 10,
+                  h - lfont->Height() - sfont->Height() - 22 );
+  DrawBox( box, BOX_RECESSED );
+
+  short xpos = (w - title_width) / 2;
+
+  lfont->Write( title, this, xpos + 3, 8, view->GetBGPen() );
+  lfont->Write( title, this, xpos, 5, view->GetFGPen() );
+
+  if (connecting) {
+    const char *connect = MSG( server ? MSG_NET_WAITING_CLIENT : MSG_NET_CONNECTING );
+    sfont->Write( connect, this, (w - sfont->TextWidth( connect )) / 2,
+                  box.y + (box.h - sfont->Height()) / 2 );
+  }
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NetworkSetupWindow::WidgetActivated
+// DESCRIPTION: When the user pushes the 'OK' button, check whether we
+//              have the necessary information.
+// PARAMETERS : button - calling widget
+//              win    - pointer to active window
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status NetworkSetupWindow::WidgetActivated( Widget *button, Window *win ) {
+  GUI_Status rc = GUI_OK;
+
+  if ((!server && (address->String() == NULL)) ||
+      (port->String() == NULL)) {
+    Audio::PlaySfx( Audio::SND_GUI_ERROR, 0 );
+  } else {
+    rc = (GUI_Status)button->ID();
+  }
+
+ return rc;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NetworkSetupWindow::Cancelled
+// DESCRIPTION: Check whether the user clicked the 'Cancel' button.
+// PARAMETERS : -
+// RETURNS    : TRUE if button was activated, FALSE otherwise
+////////////////////////////////////////////////////////////////////////
+
+bool NetworkSetupWindow::Cancelled( void ) {
+  GUI_Status peek;
+  bool abort = false;
+
+  do {
+    SDL_Event event;
+    peek = view->PeekEvent( event );
+    if ( (peek != GUI_NONE) &&
+         (HandleEvent( event ) == GUI_CLOSE) ) abort = true;
+  } while ( peek != GUI_NONE );
+
+  return abort;
+}
+
+#endif  // !DISABLE_NETWORK
