@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "textbox.h"
 #include "button.h"
 #include "mapwindow.h"
+#include "network.h"
 
 class TitleWindow : public Window {
 public:
@@ -48,7 +49,9 @@ public:
 private:
   void CompleteFilesList( TLWList &list );
   Mission *LoadMission( const char *filename, bool full = true ) const;
+  GUI_Status StartGame( const char *filename );
   void Rebuild( void );
+  short AskForSide( Mission &m ) const;
 
   CycleWidget *gtypewidget;
   CycleWidget *mtypewidget;
@@ -79,7 +82,9 @@ protected:
 
   const Rect &GetBounds( void ) const { return clientarea; }
 
-  static const short B_ID_OK;
+  enum {
+    B_ID_OK = 1
+  };
 
 private:
   Rect clientarea;  // the part subclasses may draw into
@@ -130,6 +135,33 @@ private:
   Game *game;
 };
 
+class KeyboardOptionsWindow : public GenericOptionsWindow {
+public:
+  KeyboardOptionsWindow( View *view );
+
+private:
+  GUI_Status HandleEvent( const SDL_Event &event );
+  GUI_Status WidgetActivated( Widget *widget, Window *win );
+
+  void AssignKey( SDLKey key );
+  void RebuildKeyMap( void );
+
+  TextListWidget *fncwidget;
+  List functions;
+  int last;
+  Window *request;
+};
+
+class KeyboardPressKeyWindow : public Window {
+public:
+  KeyboardPressKeyWindow( Window *parent, View *view );
+  GUI_Status HandleEvent( const SDL_Event &event )
+             { return parent->HandleEvent( event ); }
+
+private:
+  Window *parent;
+};
+
 # ifndef DISABLE_SOUND
 class SoundOptionsWindow : public GenericOptionsWindow {
 public:
@@ -139,10 +171,12 @@ public:
 private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
 
-  static const short B_ID_SFX;
-  static const short B_ID_MUSIC;
-  static const short S_ID_VOL_SFX;
-  static const short S_ID_VOL_MUSIC;
+  enum {
+    B_ID_SFX = 10,
+    B_ID_MUSIC,
+    S_ID_VOL_SFX,
+    S_ID_VOL_MUSIC
+  };
 
   SliderWidget *sfxvol;
   SliderWidget *musicvol;
@@ -151,5 +185,34 @@ private:
 };
 # endif
 
-#endif	/* _INCLUDE_INITWINDOW_H */
+#ifndef DISABLE_NETWORK
+class NetworkSetupWindow : public Window, public WidgetHook, public UserActionHook {
+public:
+  NetworkSetupWindow( bool server, View *view );
+  ~NetworkSetupWindow( void );
+
+  void Draw( void );
+
+  void SetConnecting( bool connect );
+  bool IsServer( void ) const { return server; }
+  bool Cancelled( void );
+
+  const char *GetServer( void ) const { return address->String(); }
+  const unsigned short GetPort( void ) const { return port->Number(); }
+
+private:
+  GUI_Status WidgetActivated( Widget *button, Window *win );
+
+  bool server;
+  bool connecting;
+
+  StringWidget *address;
+  NumberWidget *port;
+
+  const char *title;
+  unsigned short title_width;
+};
+#endif
+
+#endif  /* _INCLUDE_INITWINDOW_H */
 
