@@ -1,5 +1,5 @@
 // Crimson Fields -- a hex-based game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,7 +25,9 @@
 #include <iostream>
 #include <fstream>
 
-#ifndef _WIN32_WCE
+#ifdef _WIN32_WCE
+# define time(n) GetTickCount()
+#else
 # include <time.h>
 #endif
 
@@ -41,6 +43,8 @@
 #include "options.h"
 #include "strutil.h"
 #include "msgs.h"
+#include "network.h"
+#include "platform.h"
 
 // global vars
 Game *Gam;
@@ -50,18 +54,6 @@ Language Lang;
 // global options, can be accessed from everywhere
 Options CFOptions;
 
-struct GUIOptions {
-  short px_width;
-  short px_height;
-  short bpp;
-  bool sfx;
-  bool music;
-  unsigned char sfx_vol;
-  unsigned char music_vol;
-  unsigned long sdl_flags;
-  const char *level;
-};
-
 // local vars
 static View *display;
 
@@ -76,32 +68,30 @@ static void set_icon( const Surface &s, const Rect &icon );
 static GUI_Status event_filter( SDL_Event &event, Window *window );
 static void do_exit( void );
 
-#ifdef _WIN32_WCE
-static bool init_wince( struct GUIOptions &opts );
-# define time(n) GetTickCount()
-#endif
-
 // Start of program functions //////////////////////////////////////////
 
-#ifdef _WIN32_WCE
-int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPWSTR szCmdLine, int sw) {
+#ifdef _MSC_VER
+int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPTSTR szCmdLine, int sw) {
+  int argc = __argc;
+  char **argv = __argv;
 #else
 int main( int argc, char **argv ) {
 #endif
-  struct GUIOptions guiopts = { 800, 600, DISPLAY_BPP, true, true,
+  struct GUIOptions guiopts = { DEFAULT_RESOLUTION, DISPLAY_BPP, true, true,
                     MIX_MAX_VOLUME*3/4, MIX_MAX_VOLUME/2, SDL_HWSURFACE, NULL };
 
-#ifdef _WIN32_WCE
-  if ( !init_wince( guiopts ) ) return 0;
-#else
+  load_settings( guiopts );
+
+  if ( !platform_init( guiopts ) ) return 0;
+
   parse_options( argc, argv, guiopts );
-#endif
 
   display = init( guiopts );
-  if ( display ) {
+  if ( display && platform_setup( display ) ) {
 
     // only open intro screen if the user didn't supply a level on the command line
     int intro = 1;
+
     if ( guiopts.level ) {
       Gam = new Game( display );
       intro = Gam->Load( guiopts.level );
@@ -164,9 +154,7 @@ int main( int argc, char **argv ) {
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-// parse command line arguments
-void parse_options( int argc, char **argv, GUIOptions &opts ) {
-  load_settings( opts );
+static void parse_options( int argc, char **argv, GUIOptions &opts ) {
 
   while ( argc > 1 ) {
     --argc;
@@ -202,7 +190,7 @@ void parse_options( int argc, char **argv, GUIOptions &opts ) {
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void print_usage( char *prog ) {
+static void print_usage( char *prog ) {
   cout << "Usage: " << prog << " [options]" << endl << endl
             << "Available options:" << endl
             << "  --level <level>      load level or save file" << endl
@@ -226,7 +214,7 @@ void print_usage( char *prog ) {
 //              otherwise
 ////////////////////////////////////////////////////////////////////////
 
-View *init( GUIOptions &opts ) {
+static View *init( GUIOptions &opts ) {
   View *view = NULL;
   Font *f1 = NULL, *f2 = NULL;
   Surface *icons = NULL;
@@ -250,12 +238,23 @@ View *init( GUIOptions &opts ) {
         view = new View( opts.px_width, opts.px_height, opts.bpp,
                          opts.sdl_flags );
         if ( view->s_surface && !TTF_Init()) {	// load fonts
+          int smallsize, largesize;
+
+          // use smaller fonts on small displays to prevent text clipping
+          if ((opts.px_width <= 240) || (opts.px_height <= 320)) {
+            smallsize = CF_FONT_LOWRES_SMALL;
+            largesize = CF_FONT_LOWRES_LARGE;
+          } else {
+            smallsize = CF_FONT_SMALL;
+            largesize = CF_FONT_LARGE;
+          }
 
           datpath = get_data_dir() + CF_FONT;
           f1 = new Font();
           f2 = new Font();
-          if ( !f1->Load( datpath.c_str(), CF_FONT_SMALL ) &&
-               !f2->Load( datpath.c_str(), CF_FONT_LARGE ) )
+
+          if ( !f1->Load( datpath.c_str(), smallsize ) &&
+               !f2->Load( datpath.c_str(), largesize ) )
             ok = true;
           else cerr << "Error: Couldn't load font " << datpath << endl;
         } else cerr << "Error: Couldn't set video mode (" << SDL_GetError() << ')' << endl;
@@ -269,11 +268,13 @@ View *init( GUIOptions &opts ) {
     SDL_WM_SetCaption( PROGRAMNAME, PROGRAMNAME );
     SDL_EnableKeyRepeat( SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL );
 
-    srand( time(0) );				// initialize random number generator
+    srand( time(0) );        // initialize random number generator
 
-    Audio::InitSfx( opts.sfx, opts.sfx_vol );	// load sound effects
+    Audio::InitSfx( opts.sfx, opts.sfx_vol );  // load sound effects
     Audio::InitMusic( opts.music, opts.music_vol );
 
+    Network::Init();        // initialize networking subsystem
+
     for ( int i = 0; i <= XP_MAX_LEVEL; ++i )
       Images[ICON_XP_BASE+i] = new Image( icons, 64 + i * XP_ICON_WIDTH, 32, XP_ICON_WIDTH, XP_ICON_HEIGHT );
 
@@ -306,7 +307,7 @@ View *init( GUIOptions &opts ) {
 // RETURNS    : TRUE on success, FALSE otherwise
 ////////////////////////////////////////////////////////////////////////
 
-bool init_locale( void ) {
+static bool init_locale( void ) {
   bool rc = false, retry = false;
 
   const string ldir( get_locale_dir() );
@@ -354,7 +355,7 @@ bool init_locale( void ) {
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void load_settings( GUIOptions &opts ) {
+static void load_settings( GUIOptions &opts ) {
   string crimsonrc( get_config_dir() );
   crimsonrc.append( CRIMSONRC );
 
@@ -391,6 +392,17 @@ void load_settings( GUIOptions &opts ) {
             int rep = atoi(val);
             CFOptions.SetTurnReplay( rep != 0 );
             CFOptions.SetQuickReplay( rep == 1 );
+          } else if ( !strncmp( linebuf, "keymap", 6 ) ) {
+            SDLKey key;
+            char *endptr;
+
+            for (int i = 0; i < KEYBIND_COUNT && *val; ++i) {
+              key = (SDLKey)strtoul( val, &endptr, 10 );
+              if ((endptr == NULL) || (*endptr != ';'))
+                break;
+              CFOptions.SetKeyBinding( (KeyBinding)i, key );
+              val = endptr + 1;
+            }
           } else cerr << "Warning: unrecognized config option in line " << line << endl;
         }
       }
@@ -406,7 +418,8 @@ void load_settings( GUIOptions &opts ) {
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void save_settings( View *display ) {
+static void save_settings( View *display ) {
+  const SDLKey *keymap = CFOptions.GetKeyBindings();
   string crimsonrc( get_config_dir() );
   crimsonrc.append( CRIMSONRC );
 
@@ -424,6 +437,11 @@ void save_settings( View *display ) {
     file << StringUtil::strprintf("showreplay %d",
             CFOptions.GetTurnReplay() ? (CFOptions.GetQuickReplay() ? 1 : 2) : 0 ) << '\n';
 
+    file << "keymap ";
+    for (int i = 0; i < KEYBIND_COUNT; ++i)
+      file << keymap[i] << ';';
+    file << '\n';
+
     const vector<string> &maps = CFOptions.GetUnlockedMaps();
     for ( vector<string>::const_iterator i = maps.begin(); i != maps.end(); ++i )
       file << "unlock " << *i << '\n';
@@ -440,7 +458,7 @@ void save_settings( View *display ) {
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void set_icon( const Surface &s, const Rect &icon ) {
+static void set_icon( const Surface &s, const Rect &icon ) {
   Surface is;
   is.Create( icon.Width(), icon.Height(), DISPLAY_BPP, 0 );
   is.SetColorKey( s.GetColorKey() );
@@ -462,7 +480,7 @@ void set_icon( const Surface &s, const Rect &icon ) {
 //              silently drop it.
 ////////////////////////////////////////////////////////////////////////
 
-GUI_Status event_filter( SDL_Event &event, Window *window ) {
+static GUI_Status event_filter( SDL_Event &event, Window *window ) {
   GUI_Status rc = GUI_OK;
 
   if ( event.type == SDL_KEYDOWN ) {
@@ -473,42 +491,15 @@ GUI_Status event_filter( SDL_Event &event, Window *window ) {
       Audio::ToggleSfxState();
       break;
     default:
-      rc = GUI_OK;            // send to windows
+      if ( event.key.keysym.sym == CFOptions.GetKeyBindings()[KEYBIND_MINIMIZE] )
+        SDL_WM_IconifyWindow();
+      else rc = GUI_OK;            // send to windows
     }
   } else if ( event.type == SDL_QUIT ) do_exit();
 
   return rc;
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : init_wince
-// DESCRIPTION: Initialize on Windows CE.
-// PARAMETERS : opts - program options buffer (containing defaults)
-// RETURNS    : TRUE if initialization succeeded, FALSE otherwise
-////////////////////////////////////////////////////////////////////////
-
-#ifdef _WIN32_WCE
-bool init_wince( struct GUIOptions &opts ) {
-  HWND hwnd = FindWindow( _T("SDL_app"), _T(PROGRAMNAME) );
-  if ( hwnd ) {
-    SetForegroundWindow( hwnd );
-    ShowWindow( hwnd, SW_SHOW );
-    return false;
-  }
-
-  load_settings( opts );
-
-  unsigned short maxw = GetSystemMetrics(SM_CXSCREEN); 
-  unsigned short maxh = GetSystemMetrics(SM_CYSCREEN); 
-
-  if( opts.px_width > maxw ) opts.px_width = maxw;
-  if( opts.px_height > maxh ) opts.px_height = maxh;
-
-  opts.sdl_flags |= SDL_FULLSCREEN;
-  return true;
-}
-#endif
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : do_exit
 // DESCRIPTION: Free all resources and exit the program.
@@ -516,14 +507,18 @@ bool init_wince( struct GUIOptions &opts ) {
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void do_exit( void ) {
+static void do_exit( void ) {
   delete Gam;
 
+  platform_dispose();
+
   if ( display ) {
     save_settings( display );
     delete display;
   }
 
+  Network::Shutdown();
+
   Audio::ShutdownSfx();
   Audio::ShutdownMusic();
 
@@ -531,6 +526,9 @@ void do_exit( void ) {
 
   TTF_Quit();
   SDL_Quit();
+
+  platform_shutdown();
+
   exit( 0 );
 }
 
