@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ Map::~Map( void ) {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int Map::Load( File &file ) {
+int Map::Load( MemBuffer &file ) {
   m_w = file.Read16();
   m_h = file.Read16();
 
@@ -81,14 +81,15 @@ int Map::Load( File &file ) {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int Map::Save( File &file ) const {
+int Map::Save( MemBuffer &file ) const {
+  int rc;
   file.Write16( m_w );
-  file.Write16( m_h );
+  rc = file.Write16( m_h );
 
   unsigned short size = m_w * m_h;
-  for ( int i = 0; i < size; ++i )
-    if ( file.Write16( m_data[i] ) != 1 ) return -1;
-  return 0;
+  for ( int i = 0; i < size && !rc; ++i )
+    rc = file.Write16( m_data[i] );
+  return rc;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -294,7 +295,7 @@ signed char Map::MoveCost( const Unit *u, const Point &src,
                            const Point &dst, unsigned short &state ) const {
   signed char cost;
   const TerrainType *type = HexType( dst );
-  bool hexok = type->tt_type & u->Terrain();
+  bool hexok = (type->tt_type & u->Terrain()) != 0;
 
   state = 0;
 
