@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -135,16 +135,17 @@ void MapWindow::MoveHex( unsigned short img, const TileSet &tiles,
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : MapWindow::FadeInHex
+// NAME       : MapWindow::FadeHex
 // DESCRIPTION: Fade in a hex image (a unit or terrain image).
 // PARAMETERS : img  - tile identifier for the hex image
 //              hex  - destination hex position
+//              in   - whether to fade in or out
 //              unit - if TRUE paint a unit image, otherwise the
 //                     respective terrain
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void MapWindow::FadeInHex( unsigned short img, const Point &hex, bool unit ) {
+void MapWindow::FadeHex( unsigned short img, const Point &hex, bool in, bool unit ) {
   unsigned long firstticks, oldticks, ticks;
   Point pos = mview->Hex2Pixel( hex );
   const Rect dest( pos.x, pos.y, mview->TileWidth(), mview->TileHeight() );
@@ -168,8 +169,10 @@ void MapWindow::FadeInHex( unsigned short img, const Point &hex, bool unit ) {
   firstticks = oldticks = SDL_GetTicks();
   do {
     ticks = SDL_GetTicks() - firstticks;
+
     int alpha = SDL_ALPHA_OPAQUE * ticks / speed;
     if ( alpha > SDL_ALPHA_OPAQUE ) alpha = SDL_ALPHA_OPAQUE;
+    if ( !in ) alpha = SDL_ALPHA_OPAQUE - alpha;
 
     tile.SetAlpha( alpha, SDL_SRCALPHA );
     mview->DrawTerrain( terrain, this, pos.x, pos.y, dest );
