@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -70,16 +70,18 @@ public:
   void MoveHex( unsigned short img, const TileSet &tiles, const Point &hex1,
                 const Point &hex2, unsigned short speed );
   Point MoveCursor( Direction dir );
+  void FadeOutUnit( unsigned short img, const Point &hex )
+       { FadeHex( img, hex, false, true ); }
   void FadeInUnit( unsigned short img, const Point &hex )
-       { FadeInHex( img, hex, true ); }
+       { FadeHex( img, hex, true, true ); }
   void FadeInTerrain( unsigned short img, const Point &hex )
-       { FadeInHex( img, hex, false ); }
+       { FadeHex( img, hex, true, false ); }
 
   void DisplayHex( const Point &hex );
   void BoxAvoidHexes( Rect &rect, const Point &hex1, const Point &hex2 ) const;
 
 private:
-  void FadeInHex( unsigned short img, const Point &hex, bool unit );
+  void FadeHex( unsigned short img, const Point &hex, bool in, bool unit );
 
   MapView *mview;
   Panel *panel;
