@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ Mission::~Mission( void ) {
 // RETURNS    : 0 on success, non-null otherwise
 ////////////////////////////////////////////////////////////////////////
 
-int Mission::Load( File &file ) {
+int Mission::Load( MemBuffer &file ) {
   int rc = -1;
 
   // read game info
@@ -143,8 +143,7 @@ int Mission::Load( File &file ) {
 
     rc = 0;
   } else
-    cerr << "Warning: Couldn't load " << file.Name()
-         << " (incompatible format)" << endl;
+    cerr << "Warning: invalid header or version" << endl;
 
   return rc;
 }
@@ -157,7 +156,7 @@ int Mission::Load( File &file ) {
 // RETURNS    : pointer to loaded unit or NULL on error
 ////////////////////////////////////////////////////////////////////////
 
-Unit *Mission::LoadUnit( File &file, bool dummy /* = false */ ) {
+Unit *Mission::LoadUnit( MemBuffer &file, bool dummy /* = false */ ) {
   unsigned char tid, pid;
   const UnitType *type;
   Player *p = 0;
@@ -185,7 +184,7 @@ Unit *Mission::LoadUnit( File &file, bool dummy /* = false */ ) {
 // RETURNS    : 0 on success, non-null otherwise
 ////////////////////////////////////////////////////////////////////////
 
-int Mission::QuickLoad( File &file ) {
+int Mission::QuickLoad( MemBuffer &file ) {
   int rc = -1;
 
   // read game info
@@ -221,7 +220,7 @@ int Mission::QuickLoad( File &file ) {
 // RETURNS    : 0 on successful write, non-zero otherwise
 ////////////////////////////////////////////////////////////////////////
 
-int Mission::Save( File &file ) {
+int Mission::Save( MemBuffer &file ) {
   // save game info
   file.Write32( FID_MISSION );
   file.Write8( FILE_VERSION );
@@ -262,7 +261,7 @@ int Mission::Save( File &file ) {
   // make two passes through the list, first saving all unsheltered
   // units (which are possibly transports carrying other units), and
   // all sheltered units in the second run
-  file.Write16( units.CountNodes() ); 
+  file.Write16( units.CountNodes() );
   const Unit *u;
   for ( u = static_cast<Unit *>(units.Head());
         u; u = static_cast<Unit *>(u->Next()) ) {  // make sure transports are
@@ -321,7 +320,7 @@ Unit *Mission::CreateUnit( unsigned char type, Player &p,
   units.AddTail( u );
   map.SetUnit( u, pos );
 
-  if ( history ) history->RecordUnitEvent( *u );
+  if ( history ) history->RecordUnitEvent( *u, History::HIST_UEVENT_CREATE );
 
   u->SetFlags( U_DONE );     // can't move on the first turn
 
