@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -35,10 +35,10 @@ public:
   Mission( void ) { history = 0; }
   ~Mission( void );
 
-  int Load( File &file );
-  Unit *LoadUnit( File &file, bool dummy = false );
-  int QuickLoad( File &file );
-  int Save( File &file );
+  int Load( MemBuffer &file );
+  Unit *LoadUnit( MemBuffer &file, bool dummy = false );
+  int QuickLoad( MemBuffer &file );
+  int Save( MemBuffer &file );
 
   Map &GetMap( void ) { return map; }
   TerrainSet &GetTerrainSet( void ) { return terrain_set; }
