@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -332,9 +332,7 @@ short Path::FinalizePath( const Unit *u, const PathNode &last ) const {
 // RETURNS    : estimated cost to destination
 ////////////////////////////////////////////////////////////////////////
 
-inline unsigned short MoveShader::ETA( const Point &p ) const {
-  return 1;
-}
+// inline unsigned short MoveShader::ETA( const Point &p ) const;
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MoveShader::StopSearch
@@ -345,9 +343,7 @@ inline unsigned short MoveShader::ETA( const Point &p ) const {
 // RETURNS    : TRUE if search aborted, FALSE otherwise
 ////////////////////////////////////////////////////////////////////////
 
-inline bool MoveShader::StopSearch( const PathNode &next ) const {
-  return false;
-}
+// inline bool MoveShader::StopSearch( const PathNode &next ) const;
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MoveShader::AddNode
