@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -100,8 +100,8 @@ public:
             { Find( u, u->Position(), Point(0,0) ); }
 
 protected:
-  unsigned short ETA( const Point &p ) const;
-  bool StopSearch( const PathNode &next ) const;
+  unsigned short ETA( const Point &p ) const { return 1; }
+  bool StopSearch( const PathNode &next ) const { return false; }
   virtual bool AddNode( const Unit *u, const PathNode &from,
                         PathNode &to ) const;
   virtual short FinalizePath( const Unit *u, const PathNode &last ) const;
