@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-int Player::Load( File &file ) {
+int Player::Load( MemBuffer &file ) {
   p_id = file.Read8();
   p_type = file.Read8();
   p_success = file.Read8();
@@ -51,6 +51,7 @@ int Player::Load( File &file ) {
   p_col_dark.b = file.Read8();
 
   p_mode = MODE_IDLE;
+  p_remote = false;
   p_units = 0;
   return 0;
 }
@@ -62,7 +63,7 @@ int Player::Load( File &file ) {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int Player::Save( File &file ) const {
+int Player::Save( MemBuffer &file ) const {
   file.Write8( p_id );
   file.Write8( p_type );
   file.Write8( p_success );
