@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ Unit::Unit( const UnitType *type, Player *player, unsigned short id, const Point
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-int Unit::Load( File &file, const UnitType *type, Player *player ) {
+int Unit::Load( MemBuffer &file, const UnitType *type, Player *player ) {
   u_pos.x = file.Read16();
   u_pos.y = file.Read16();
   u_flags = file.Read32();
@@ -79,7 +79,7 @@ int Unit::Load( File &file, const UnitType *type, Player *player ) {
 // RETURNS    : 0 on succes, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int Unit::Save( File &file ) const {
+int Unit::Save( MemBuffer &file ) const {
   file.Write8( u_type->ID() );
   file.Write8( u_player ? u_player->ID() : PLAYER_NONE );
 
