@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -54,8 +54,8 @@ public:
   Unit( void ) : MapObject(MO_UNIT) {}
   Unit( const UnitType *type, Player *player, unsigned short id, const Point &pos );
 
-  virtual int Load( File &file, const UnitType *type, Player *player );
-  virtual int Save( File &file ) const;
+  virtual int Load( MemBuffer &file, const UnitType *type, Player *player );
+  virtual int Save( MemBuffer &file ) const;
 
   unsigned short BaseImage( void ) const { return u_type->Image() + u_player->ID() * 6; }
   unsigned short BuildCost( void ) const { return u_type->Cost(); }
@@ -72,7 +72,8 @@ public:
   unsigned short Terrain( void ) const { return u_type->Terrain(); }
   const UnitType *Type( void ) const { return u_type; }
   unsigned char WeaponRange( const Unit *u ) const;
-  unsigned char XPLevel( void ) const { return u_xp/XP_PER_LEVEL; }
+  unsigned char XP( void ) const { return u_xp; }
+  unsigned char XPLevel( void ) const { return XP()/XP_PER_LEVEL; }
 
   void AwardXP( unsigned char xp );
   void SetOwner( Player *player );
