@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,19 +27,19 @@
 #include "strutil.h"
 #include "msgs.h"
 
-#define ICON_MINI_WIDTH		15
-#define ICON_MINI_HEIGHT	15
-#define ICON_MINI_REPAIR1_X	93
-#define ICON_MINI_REPAIR1_Y	46
-#define ICON_MINI_REPAIR2_X	93
-#define ICON_MINI_REPAIR2_Y	46
-#define ICON_MINI_BUILD1_X	108
-#define ICON_MINI_BUILD1_Y	46
-#define ICON_MINI_BUILD2_X	108
-#define ICON_MINI_BUILD2_Y	46
+#define ICON_MINI_WIDTH    15
+#define ICON_MINI_HEIGHT  15
+#define ICON_MINI_REPAIR1_X  93
+#define ICON_MINI_REPAIR1_Y  46
+#define ICON_MINI_REPAIR2_X  93
+#define ICON_MINI_REPAIR2_Y  46
+#define ICON_MINI_BUILD1_X  108
+#define ICON_MINI_BUILD1_Y  46
+#define ICON_MINI_BUILD2_X  108
+#define ICON_MINI_BUILD2_Y  46
 
-#define CW_MODE_NORMAL		0
-#define CW_MODE_PRODUCTION	1
+#define CW_MODE_NORMAL    0
+#define CW_MODE_PRODUCTION  1
 
 // node class used by the UnitListWidget
 class ULWNode : public Node {
@@ -71,16 +71,16 @@ private:
 
 
 // button IDs for the container hook
-#define CH_BUTTON_REPAIR	0
-#define CH_BUTTON_BUILD		1
+#define CH_BUTTON_REPAIR  0
+#define CH_BUTTON_BUILD    1
 
 #define CH_BUTTON_REPAIR_CONFIRM 2
-#define CH_LIST_UNITS		3
+#define CH_LIST_UNITS    3
 
 // widget IDs for the UnitLoadWindow
-#define ULW_WIDGET_LIST		1
-#define ULW_WIDGET_OK		2
-#define ULW_WIDGET_CRYSTALS	3
+#define ULW_WIDGET_LIST    1
+#define ULW_WIDGET_OK    2
+#define ULW_WIDGET_CRYSTALS  3
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : UnitListWidget::UnitListWidget
@@ -342,7 +342,7 @@ void ContainerWindow::DrawCrystals( void ) {
              tlen + ICON_WIDTH + 5, sfont->Height() );
   DrawBack( rect );
   sfont->Write( buf, this, rect.x + 30, rect.y );
-} 
+}
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : ContainerWindow::HandleEvent
@@ -362,7 +362,7 @@ GUI_Status ContainerWindow::HandleEvent( const SDL_Event &event ) {
     rc = SelectNode( static_cast<ULWNode *>( listwidget->Selected() ) );
   } else rc = Window::HandleEvent( event );
   return rc;
-} 
+}
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : ContainerWindow::SwitchMode
@@ -424,6 +424,7 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
   }
 
   mode = newmode;
+  last_selected = NULL;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -458,7 +459,7 @@ GUI_Status ContainerWindow::SelectNode( ULWNode *node ) {
         ulw->selected = false;
         normal.AddTail( ulw );
 
-	b->SetCrystals( b->Crystals() - u->BuildCost() );
+        b->SetCrystals( b->Crystals() - u->BuildCost() );
         DrawCrystals();
         Show();
         SwitchMode( CW_MODE_NORMAL );
@@ -511,11 +512,14 @@ GUI_Status ContainerWindow::WidgetActivated( Widget *widget, Window *win ) {
     Audio::PlaySfx( Audio::SND_GAM_REPAIR, 0 );
     Unit *u = static_cast<ULWNode *>(listwidget->Selected())->unit;
     c->SetCrystals( c->Crystals() - CRYSTALS_REPAIR );
+    if ( Gam->GetMission()->GetHistory() )
+      Gam->GetMission()->GetHistory()->RecordUnitEvent( *u, History::HIST_UEVENT_REPAIR );
+
     u->Repair();
-    listwidget->Draw();					// update list display
-    DrawCrystals();					// update crystals
+    listwidget->Draw();      // update list display
+    DrawCrystals();          // update crystals
     Show();
-    rc = GUI_CLOSE;					// close the request window
+    rc = GUI_CLOSE;          // close the request window
     break; }
 
   case CH_LIST_UNITS: {
@@ -546,18 +550,21 @@ GUI_Status ContainerWindow::WidgetActivated( Widget *widget, Window *win ) {
 // NAME       : UnitLoadWindow::UnitLoadWindow
 // DESCRIPTION: This window is used to ask the player which units to
 //              load when he moves a transport out of a container.
-// PARAMETERS : t    - transport to be moved
-//              c    - unit container the transport is moving out of
-//              uset - unit set used
-//              tset - terrain set used
-//              view - pointer to the view
+// PARAMETERS : t       - transport to be moved
+//              c       - unit container the transport is moving out of
+//              uset    - unit set used
+//              tset    - terrain set used
+//              history - for event recording
+//              view    - pointer to the view
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
 UnitLoadWindow::UnitLoadWindow( Transport &t, UnitContainer &c,
-        const UnitSet &uset, const TerrainSet &tset, View *view ) :
+        const UnitSet &uset, const TerrainSet &tset,
+        History *history, View *view ) :
         Window( WIN_CENTER, view ), t(t), c(c), last_selected(0),
-        fullslots(t.FullSlots()), crystals(t.Crystals()), have_units(false) {
+        fullslots(t.FullSlots()), crystals(t.Crystals()),
+        have_units(false), history(history) {
   // calculate window dimensions
   short width = uset.TileWidth() + XP_ICON_WIDTH + DEFAULT_SLIDER_SIZE +
                 MAX(sfont->Width() * 24, tset.TileWidth() * 4) + 20;
@@ -732,8 +739,16 @@ GUI_Status UnitLoadWindow::WidgetActivated( Widget *widget, Window *win ) {
         c.RemoveUnit( node->unit );
         node->unit->SetFlags( U_MOVED|U_DONE );
         t.InsertUnit( node->unit );
+
+        if ( history )
+          history->RecordTransportEvent( c, t, *node->unit );
+
       }
     }
+
+    if ( history && (t.Crystals() != crystals) )
+      history->RecordTransportEvent( c, t, crystals - t.Crystals() );
+
     c.SetCrystals( c.Crystals() - (crystals - t.Crystals()) );
     t.SetCrystals( crystals );
     rc = (GUI_Status)ULW_WIDGET_OK;
