@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2005 Jens Granseuer
+// Copyright (C) 2002-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -55,7 +55,7 @@ Building::Building( const Point &pos, unsigned short id,
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Building::Load( File &file ) {
+int Building::Load( MemBuffer &file ) {
   b_id = file.Read16();
   b_pos.x = file.Read16();
   b_pos.y = file.Read16();
@@ -80,7 +80,7 @@ int Building::Load( File &file ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Building::Save( File &file ) const {
+int Building::Save( MemBuffer &file ) const {
   file.Write16( b_id );
   file.Write16( b_pos.x );
   file.Write16( b_pos.y );
