@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2005 Jens Granseuer
+// Copyright (C) 2002-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,33 +30,6 @@
 #include "strutil.h"
 #include "main.h"
 
-// button IDs for the context menu
-#define B_ID_MENU_BASE       200
-
-// inititalize static vars
-const short EdWindow::L_ID_UNITS      = 0;
-const short EdWindow::L_ID_TILES      = 1;
-const short EdWindow::S_ID_VERT       = 2;
-const short EdWindow::S_ID_HORIZ      = 3;
-
-const short EdWindow::B_ID_TILE_GRAB  = B_ID_MENU_BASE + 0;
-const short EdWindow::B_ID_TILE_SWAP  = B_ID_MENU_BASE + 1;
-const short EdWindow::B_ID_UNIT_INFO  = B_ID_MENU_BASE + 2;
-const short EdWindow::B_ID_UNIT_EDIT  = B_ID_MENU_BASE + 3;
-const short EdWindow::B_ID_UNIT_DEL   = B_ID_MENU_BASE + 4;
-const short EdWindow::B_ID_BLD_CREATE = B_ID_MENU_BASE + 5;
-const short EdWindow::B_ID_BLD_EDIT   = B_ID_MENU_BASE + 6;
-const short EdWindow::B_ID_BLD_DEL    = B_ID_MENU_BASE + 7;
-const short EdWindow::B_ID_EVENTS     = B_ID_MENU_BASE + 8;
-const short EdWindow::B_ID_MESSAGES   = B_ID_MENU_BASE + 9;
-const short EdWindow::B_ID_NEW        = B_ID_MENU_BASE + 10;
-const short EdWindow::B_ID_LOAD       = B_ID_MENU_BASE + 11;
-const short EdWindow::B_ID_SAVE       = B_ID_MENU_BASE + 12;
-const short EdWindow::B_ID_EXPORT     = B_ID_MENU_BASE + 13;
-const short EdWindow::B_ID_VALIDATE   = B_ID_MENU_BASE + 14;
-const short EdWindow::B_ID_SETTINGS   = B_ID_MENU_BASE + 15;
-const short EdWindow::B_ID_QUIT       = B_ID_MENU_BASE + 16;
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : EdWindow::EdWindow
 // DESCRIPTION: Create the editing window.
@@ -67,15 +40,14 @@ const short EdWindow::B_ID_QUIT       = B_ID_MENU_BASE + 16;
 
 EdWindow::EdWindow( const string &mapdir, View *view ) :
           Window( view->x, view->y, view->w, view->h, 0, view ),
+          mission(NULL),
           panel( view->w - DEFAULT_TILE_WIDTH * 4 - DEFAULT_SLIDER_SIZE - 20,
                  view->y, DEFAULT_TILE_WIDTH * 4 + DEFAULT_SLIDER_SIZE + 20, view->h ),
           mv( this,
               Rect( DEFAULT_SLIDER_SIZE, 0,
               w - panel.w - DEFAULT_SLIDER_SIZE, h - DEFAULT_SLIDER_SIZE),
-              MV_DISABLE|MV_DISABLE_FOG ), mapdir(mapdir) {
-  mode = ED_MODE_VIEW;
-  mission = NULL;
-  selected = -1;
+              MV_DISABLE|MV_DISABLE_FOG ),
+          mode(ED_MODE_VIEW), selected(-1), mapdir(mapdir) {
 
   unsigned short height = MIN( DEFAULT_TILE_HEIGHT * 6 + 4, h / 3 - 10 );
   unit_wd = new UnitWidget( L_ID_UNITS, panel.x + 5,
@@ -199,14 +171,14 @@ GUI_Status EdWindow::HandleEvent( const SDL_Event &event ) {
       case SDLK_RETURN:
         ShowContextMenu( mv.Cursor() );
 
-      case SDLK_u: {		// edit unit
+      case SDLK_u: {    // edit unit
         Unit *u = mv.GetMap()->GetUnit( mv.Cursor() );
         if ( u ) {
           EdUnitWindow *euw = new EdUnitWindow( *u, *mission, view );
           euw->SetMapView( mv );
         }
         break; }
-      case SDLK_b: {		// edit building
+      case SDLK_b: {    // edit building
         Building *b = mv.GetMap()->GetBuilding( mv.Cursor() );
         if ( b ) new EdBuildingWindow( *b, *mission, view );
         break; }
@@ -224,7 +196,18 @@ GUI_Status EdWindow::HandleEvent( const SDL_Event &event ) {
       } else if ( event.button.button == SDL_BUTTON_RIGHT ) {
          ShowContextMenu( validhex ? pos : Point(-1,-1) );
       }
+
+    } else if ( event.type == SDL_MOUSEMOTION ) {
+      // handle just like a mouse button if LMB is down and
+      // user is terraforming
+      if ( (mode == ED_MODE_TERRAIN) &&
+           (event.motion.state & SDL_BUTTON( SDL_BUTTON_LEFT )) ) {
+        Point pos;
+        if ( !mv.Pixel2Hex( event.motion.x, event.motion.y, pos ) )
+          LeftMouseButton( pos );
+      }
     }
+
   }
   return rc;
 }
@@ -514,14 +497,15 @@ GUI_Status EdWindow::WidgetActivated( Widget *widget, Window *win ) {
 
     Map *map = mv.GetMap();
 
-    if ( widget->ID() >= B_ID_MENU_BASE )
+    // context menu?
+    if ( widget->ID() >= B_ID_TILE_GRAB )
       static_cast<MenuWindow *>(win)->CloseParent();
     view->CloseWindow(win);
 
     switch ( widget->ID() ) {
     case B_ID_NEW: {
       NewMissionWindow *nmw = new NewMissionWindow( view );
-      nmw->SetHook( this );
+      nmw->SetHook( this, B_ID_NEW_MISSION_OK );
       break; }
     case B_ID_LOAD:
       LoadMission( "", true );
@@ -583,7 +567,7 @@ GUI_Status EdWindow::WidgetActivated( Widget *widget, Window *win ) {
       new EdMsgWindow( mission->GetMessages(), view );
       break;
 
-    case B_ID_NMW_CREATE: {
+    case B_ID_NEW_MISSION_OK: {
       NewMissionWindow *nmw = static_cast<NewMissionWindow *>(win);
       if ( nmw->GetMission() )
         SetNewMission( nmw->GetMission() );
