@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2005 Jens Granseuer
+// Copyright (C) 2002-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,95 +31,6 @@ extern const char *size_labels[];
 extern const char *xp_labels[];
 static const char *uselect_labels[] = { "Single Unit", "Unit Type", 0 };
 
-// initialize static class variables
-const short EdEventGenericWindow::B_ID_TITLE = 1;
-const short EdEventGenericWindow::B_ID_MSG = 2;
-const short EdEventGenericWindow::B_ID_PLAYER = 3;
-const short EdEventGenericWindow::B_ID_DEPEND = 4;
-const short EdEventGenericWindow::B_ID_DISCARD = 5;
-const short EdEventGenericWindow::B_ID_TRIGGER = 6;
-const short EdEventGenericWindow::B_ID_EVENT = 7;
-const short EdEventGenericWindow::B_ID_TITLE_OK = 8;
-const short EdEventGenericWindow::B_ID_MSG_OK = 9;
-const short EdEventGenericWindow::B_ID_DEPEND_OK = 10;
-const short EdEventGenericWindow::B_ID_DISCARD_OK = 11;
-const short EdEventGenericWindow::B_ID_DISABLED = 12;
-
-const short EdEventCreateUnitWindow::B_ID_UNIT = 1;
-const short EdEventCreateUnitWindow::B_ID_UNIT_OK = 2;
-const short EdEventCreateUnitWindow::N_ID_XPOS = 3;
-const short EdEventCreateUnitWindow::N_ID_YPOS = 4;
-const short EdEventCreateUnitWindow::B_ID_DIRECTION = 5;
-const short EdEventCreateUnitWindow::B_ID_SIZE = 6;
-const short EdEventCreateUnitWindow::B_ID_XP = 7;
-
-const short EdEventManipEventWindow::B_ID_EVENT = 1;
-const short EdEventManipEventWindow::B_ID_EVENT_OK = 2;
-const short EdEventManipEventWindow::B_ID_ACTION = 3;
-const short EdEventManipEventWindow::B_ID_DISABLED = 4;
-
-const short EdEventMiningWindow::B_ID_MINE = 1;
-const short EdEventMiningWindow::B_ID_MINE_OK = 2;
-const short EdEventMiningWindow::B_ID_ACTION = 3;
-const short EdEventMiningWindow::N_ID_CRYSTALS = 4;
-
-const short EdEventConfigureWindow::B_ID_SETTING = 1;
-const short EdEventConfigureWindow::B_ID_MSG = 2;
-const short EdEventConfigureWindow::B_ID_MSG_OK = 3;
-const short EdEventConfigureWindow::S_ID_VALUE = 4;
-
-const short EdEventResearchWindow::B_ID_UNIT = 1;
-const short EdEventResearchWindow::B_ID_UNIT_OK = 2;
-const short EdEventResearchWindow::B_ID_SHOP = 3;
-const short EdEventResearchWindow::B_ID_SHOP_OK = 4;
-const short EdEventResearchWindow::B_ID_ACTION = 5;
-
-const short EdEventScoreWindow::B_ID_MSG = 1;
-const short EdEventScoreWindow::B_ID_MSG_OK = 2;
-const short EdEventScoreWindow::B_ID_TITLE = 3;
-const short EdEventScoreWindow::B_ID_TITLE_OK = 4;
-const short EdEventScoreWindow::N_ID_SCORE = 5;
-
-const short EdEventSetHexWindow::B_ID_TILE_PREV = 1;
-const short EdEventSetHexWindow::B_ID_TILE_NEXT = 2;
-const short EdEventSetHexWindow::N_ID_XPOS = 3;
-const short EdEventSetHexWindow::N_ID_YPOS = 4;
-
-const short EdEventSetTimerWindow::B_ID_EVENT = 1;
-const short EdEventSetTimerWindow::B_ID_EVENT_OK = 2;
-const short EdEventSetTimerWindow::B_ID_OFFSET = 3;
-const short EdEventSetTimerWindow::N_ID_TIME = 4;
-
-const short EdTrigHaveCrystalsWindow::B_ID_SHOP = 1;
-const short EdTrigHaveCrystalsWindow::B_ID_SHOP_OK = 2;
-const short EdTrigHaveCrystalsWindow::B_ID_PLAYER = 3;
-const short EdTrigHaveCrystalsWindow::N_ID_CRYSTALS = 4;
-const short EdTrigHaveCrystalsWindow::B_ID_MOREORLESS = 5;
-const short EdTrigHaveCrystalsWindow::B_ID_TRANSPORTS = 6;
-
-const short EdTrigHaveShopWindow::B_ID_SHOP = 1;
-const short EdTrigHaveShopWindow::B_ID_SHOP_OK = 2;
-const short EdTrigHaveShopWindow::B_ID_PLAYER = 3;
-const short EdTrigHaveShopWindow::N_ID_TIMER = 4;
-
-const short EdTrigHaveUnitWindow::B_ID_UNIT = 1;
-const short EdTrigHaveUnitWindow::B_ID_UNIT_OK = 2;
-const short EdTrigHaveUnitWindow::B_ID_PLAYER = 3;
-const short EdTrigHaveUnitWindow::N_ID_TIMER = 4;
-
-const short EdTrigUnitDestroyedWindow::B_ID_UNIT = 1;
-const short EdTrigUnitDestroyedWindow::B_ID_UNIT_OK = 2;
-const short EdTrigUnitDestroyedWindow::B_ID_PLAYER = 3;
-const short EdTrigUnitDestroyedWindow::B_ID_USELECT = 4;
-
-const short EdTrigUnitPositionWindow::B_ID_UNIT = 1;
-const short EdTrigUnitPositionWindow::B_ID_UNIT_OK = 2;
-const short EdTrigUnitPositionWindow::N_ID_POSX = 3;
-const short EdTrigUnitPositionWindow::N_ID_POSY = 4;
-const short EdTrigUnitPositionWindow::B_ID_PLAYER = 5;
-const short EdTrigUnitPositionWindow::B_ID_USELECT = 6;
-
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : EdEventGenericWindow::EdEventGenericWindow
 // DESCRIPTION: This window allows modification of the fields common to
@@ -341,6 +252,8 @@ GUI_Status EdEventGenericWindow::WidgetActivated( Widget *widget, Window *win )
       new EdEventConfigureWindow( event, mission, view ); break;
     case EVENT_CREATE_UNIT:
       new EdEventCreateUnitWindow( event, mission, view ); break;
+    case EVENT_DESTROY_UNIT:
+      new EdEventDestroyUnitWindow( event, mission, view ); break;
     case EVENT_MANIPULATE_EVENT:
       new EdEventManipEventWindow( event, mission, view ); break;
     case EVENT_MESSAGE:
@@ -594,6 +507,81 @@ GUI_Status EdEventCreateUnitWindow::WidgetActivated( Widget *widget, Window *win
   return GUI_OK;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdEventDestroyUnitWindow::EdEventDestroyUnitWindow
+// DESCRIPTION: This window allows configuration of a DESTROY_UNIT
+//              event.
+// PARAMETERS : event   - event to edit
+//              mission - current mission
+//              view    - pointer to the window's view
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+EdEventDestroyUnitWindow::EdEventDestroyUnitWindow( Event &event,
+                      Mission &mission, View *view ) :
+      Window(WIN_CENTER, view), event(event), mission(mission) { 
+  SetSize( MIN(sfont->Width() * 30 + 15, view->Width()),
+           sfont->Height() * 4 + 50 );
+
+  Widget *wd;
+  unsigned short wdx, wdh = sfont->Height() + 8;
+  char buf[32] = "Error";
+  static const char *player_labels[] = { "Any", "One", "Two", 0 };
+
+  wd = new ButtonWidget( B_ID_UNIT, 5, 5,
+       sfont->Width() * 4 + 10, wdh, 0, "_Unit", this );
+  wd->SetHook( this );
+
+  Unit *u = mission.GetUnitByID( event.GetData(0) );
+  if ( u ) sprintf( buf, "%s (%d)", u->Name(), u->ID() );
+
+  wdx = wd->LeftEdge() + wd->Width() + 5;
+  e_unit = new StringWidget( 0, wdx, 5, w - wdx - 5, wdh,
+           buf, 30, WIDGET_STR_CONST, NULL, this );
+
+  wdx = sfont->Width() * 20 + 10;
+  wd = new CycleWidget( B_ID_PLAYER, wdx, wd->Height() + 10,
+       w - wdx - 5, wdh, 0, "Controlled by _Player",
+       event.GetData(1) + 1, player_labels, this );
+  wd->SetHook( this );
+
+  new ButtonWidget( GUI_CLOSE, 1, h - wdh - 1, w - 2, wdh,
+                    WIDGET_DEFAULT, "_OK", this );
+
+  Draw();
+  Show();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdEventDestroyUnitWindow::WidgetActivated
+// DESCRIPTION: React to user actions.
+// PARAMETERS : widget - activated widget
+//              win    - window containing the widget
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status EdEventDestroyUnitWindow::WidgetActivated( Widget *widget, Window *win ) {
+
+  switch( widget->ID() ) {
+  case B_ID_UNIT:
+    TLWListBuilder::BuildUnitList( mission.GetUnits(), tlist );
+    new ListSelectWindow( "Select unit", tlist,
+        event.GetData(0), false, this, B_ID_UNIT_OK, view );
+    break;
+  case B_ID_UNIT_OK: {
+    TLWNode *n = static_cast<ListSelectWindow *>(win)->Selected();
+    event.SetData( 0, n ? n->ID() : -1 );
+    e_unit->SetString( n ? n->Name() : "Error" );
+    tlist.Clear();
+    break; }
+  case B_ID_PLAYER:
+    event.SetData( 1, static_cast<CycleWidget *>(widget)->GetValue() - 1 );
+    break;
+  }
+
+  return GUI_OK;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : EdEventManipEventWindow::EdEventManipEventWindow
 // DESCRIPTION: This window allows configuration of a MANIPULATE_EVENT
@@ -1467,6 +1455,7 @@ EdTrigHaveUnitWindow::EdTrigHaveUnitWindow( Event &event,
   wd = new CycleWidget( B_ID_PLAYER, wdx, wd->Height() + 10,
        w - wdx - 5, wdh, 0, "Controlled by _Player",
        event.GetTData(1), player2_labels, this );
+  wd->SetHook( this );
 
   wd = new NumberWidget( N_ID_TIMER,
        sfont->Width() * 8 + 10, wd->TopEdge() + wd->Height() + 5,
@@ -1829,14 +1818,14 @@ EdTrigHandicapWindow::EdTrigHandicapWindow( Event &event, View *view ) :
 
   wd = new CheckboxWidget( HANDICAP_P1, 5, wd->TopEdge() + wd->Height() + 5,
        DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE,
-       event.GetTData(0) & HANDICAP_P1,
+       (event.GetTData(0) & HANDICAP_P1) != 0,
        WIDGET_STYLE_GFX|WIDGET_STYLE_NOBORDER|WIDGET_ALIGN_RIGHT,
        "Player _1 handicapped", this );
   wd->SetHook( this );
 
   wd = new CheckboxWidget( HANDICAP_P2, 5, wd->TopEdge() + wd->Height() + 5,
        DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE,
-       event.GetTData(0) & HANDICAP_P2,
+       (event.GetTData(0) & HANDICAP_P2) != 0,
        WIDGET_STYLE_GFX|WIDGET_STYLE_NOBORDER|WIDGET_ALIGN_RIGHT,
        "Player _2 handicapped", this );
   wd->SetHook( this );
