@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2005 Jens Granseuer
+// Copyright (C) 2002-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,75 +30,6 @@
 #include "uiaux.h"
 #include "mapgen.h"
 
-// initialize static class variables
-const short ListExchangeWindow::B_ID_OK = 1;
-const short ListExchangeWindow::B_ID_RIGHT = 2;
-const short ListExchangeWindow::B_ID_LEFT = 3;
-
-const short NewMissionWindow::B_ID_UNITSET = 1;
-const short NewMissionWindow::B_ID_TILESET = 2;
-const short NewMissionWindow::B_ID_UNITSET_OK = 3;
-const short NewMissionWindow::B_ID_TILESET_OK = 4;
-const short NewMissionWindow::B_ID_RANDOM = 5;
-
-const short EdMissionSetupWindow::B_ID_NAME = 0;
-const short EdMissionSetupWindow::B_ID_INFO = 1;
-const short EdMissionSetupWindow::B_ID_PLAYER1 = 2;
-const short EdMissionSetupWindow::B_ID_PLAYER2 = 3;
-const short EdMissionSetupWindow::B_ID_BRIEF1 = 4;
-const short EdMissionSetupWindow::B_ID_BRIEF2 = 5;
-const short EdMissionSetupWindow::B_ID_CAMPAIGN_NAME = 6;
-const short EdMissionSetupWindow::B_ID_CAMPAIGN_INFO = 7;
-const short EdMissionSetupWindow::B_ID_SHOWMSG = 8;
-const short EdMissionSetupWindow::B_ID_SET = 9;
-const short EdMissionSetupWindow::S_ID_SEQUEL = 10;
-const short EdMissionSetupWindow::B_ID_PLAYERS = 11;
-const short EdMissionSetupWindow::S_ID_MUSIC = 12;
-const short EdMissionSetupWindow::B_ID_CAMPAIGN = 13;
-const short EdMissionSetupWindow::B_ID_SKIRMISH = 14;
-
-const short EdBuildingWindow::B_ID_PLAYER = 1;
-const short EdBuildingWindow::B_ID_WORKSHOP = 2;
-const short EdBuildingWindow::B_ID_FACTORY = 3;
-const short EdBuildingWindow::B_ID_CRYSTALS = 4;
-const short EdBuildingWindow::B_ID_OK = 5;
-const short EdBuildingWindow::B_ID_NAME = 6;
-const short EdBuildingWindow::B_ID_NAME_OK = 7;
-const short EdBuildingWindow::B_ID_UNITS = 8;
-const short EdBuildingWindow::N_ID_MAXCRYSTALS = 9;
-const short EdBuildingWindow::B_ID_FACTORY_UNITS = 10;
-
-const short EdUnitWindow::B_ID_PLAYER = 1;
-const short EdUnitWindow::B_ID_DIRECTION = 2;
-const short EdUnitWindow::B_ID_SIZE = 3;
-const short EdUnitWindow::B_ID_XP = 4;
-const short EdUnitWindow::N_ID_CRYSTALS = 5;
-const short EdUnitWindow::B_ID_UNITS = 6;
-const short EdUnitWindow::B_ID_OK = 7;
-
-const short EdUnitsWindow::B_ID_NEW = 1;
-const short EdUnitsWindow::B_ID_EDIT = 2;
-const short EdUnitsWindow::B_ID_DELETE = 3;
-const short EdUnitsWindow::B_ID_OK = 4;
-
-const short EdEventsWindow::B_ID_NEW = 1;
-const short EdEventsWindow::B_ID_EDIT = 2;
-const short EdEventsWindow::B_ID_DELETE = 3;
-const short EdEventsWindow::B_ID_OK = 4;
-
-const short SelectEventWindow::B_ID_TRIGGER = 1;
-const short SelectEventWindow::B_ID_OK = 2;
-const short SelectEventWindow::B_ID_CANCEL = 3;
-
-const short SelectUnitWindow::B_ID_CANCEL = 1;
-
-const short EdMsgWindow::B_ID_LANG_NEW = 1;
-const short EdMsgWindow::B_ID_LANG_DELETE = 2;
-const short EdMsgWindow::B_ID_MSG_ADD = 3;
-const short EdMsgWindow::B_ID_MSG_MODIFY = 4;
-const short EdMsgWindow::L_ID_LANGS = 5;
-const short EdMsgWindow::L_ID_MSGS = 6;
-
 // static vars
 const char *player2_labels[] = { "One", "Two", 0 };
 const char *player3_labels[] = { "One", "Two", "Neutral", 0 };
@@ -111,6 +42,10 @@ const char *size_labels[] = { "1", "2", "3", "4", "5", "6", 0 };
 extern const char *event_labels[];
 extern const char *etrigger_labels[];
 
+// non-private widget ID for the SelectUnitWindow
+#define B_ID_SUW_OK      102
+
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : ListExchangeWindow::ListExchangeWindow
 // DESCRIPTION: This window displays two TLWList objects. The user can
@@ -313,7 +248,7 @@ GUI_Status ListSelectWindow::WidgetActivated( Widget *widget, Window *win ) {
 ////////////////////////////////////////////////////////////////////////
 
 NewMissionWindow::NewMissionWindow( View *view ) :
-   Window(WIN_CENTER, view), mission(0), hook(0) {
+   Window(WIN_CENTER, view), mission(0), hook(0), ok_id(B_ID_OK) {
   // calculate window dimensions
   Font *f = SmallFont();
   unsigned short width = MIN( f->Width() * 38 + 20, view->Width() );
@@ -371,7 +306,7 @@ NewMissionWindow::NewMissionWindow( View *view ) :
                   WIDGET_HSCROLL|WIDGET_ALIGN_LEFT, "_Roughness", this );
 
   yoff = h - f->Height() - 15;
-  btn = new ButtonWidget( B_ID_NMW_CREATE, 10, yoff,
+  btn = new ButtonWidget( B_ID_OK, 10, yoff,
         f->Width() * 10, f->Height() + 10, 0, "_OK", this );
   btn->SetHook( this );
 
@@ -452,7 +387,7 @@ GUI_Status NewMissionWindow::WidgetActivated( Widget *button, Window *win ) {
     gen_roughness->Draw();
     Show();
     break;
-  case B_ID_NMW_CREATE:
+  case B_ID_OK:
     if ( hook ) {
       mission = NewMission( GetMapSize(),
                 GetTileSet(), GetUnitSet() );
@@ -462,7 +397,10 @@ GUI_Status NewMissionWindow::WidgetActivated( Widget *button, Window *win ) {
           gen.Generate( mission->GetMap(),
               gen_water->Level(), gen_roughness->Level() );
         }
+        // use the button ID supplied by the caller
+        button->SetID( ok_id );
         hook->WidgetActivated( button, this );
+        button->SetID( B_ID_OK );
       }
     }
   }
@@ -519,7 +457,7 @@ Mission *NewMissionWindow::NewMission( const Point &size,
 ////////////////////////////////////////////////////////////////////////
 
 EdMissionSetupWindow::EdMissionSetupWindow( Mission &mission, View *view ) :
-       Window(WIN_CENTER, view), mission(mission) { 
+       Window(WIN_CENTER, view), mission(mission) {
   // calculate dimensions
   SetSize( MIN(sfont->Width() * 40 + 20, view->Width()),
            MIN(sfont->Height() * 11 + 110, view->Height()) );
@@ -738,7 +676,7 @@ GUI_Status EdMissionSetupWindow::WidgetActivated( Widget *widget, Window *win )
 ////////////////////////////////////////////////////////////////////////
 
 EdBuildingWindow::EdBuildingWindow( Building &bld, Mission &mission, View *view ) :
-                  Window(WIN_CENTER, view), b(bld), mission(mission) { 
+                  Window(WIN_CENTER, view), b(bld), mission(mission) {
   // calculate dimensions
   SetSize( MIN(sfont->Width() * 50, view->Width()),
            MIN(sfont->Height() * 12, view->Height()) );
@@ -882,8 +820,7 @@ GUI_Status EdBuildingWindow::WidgetActivated( Widget *widget, Window *win ) {
     } else b.SetUnitProduction( 0 );
     break;
   case N_ID_MAXCRYSTALS:
-    if ( b_crystals->Number() > b_maxcrystals->Number() )
-      b_crystals->SetNumber( b_maxcrystals->Number() );
+    b_crystals->SetMax( b_maxcrystals->Number() );
     break;
   case B_ID_NAME:
     new ListSelectWindow( "Select name", msg_list,
@@ -918,7 +855,7 @@ GUI_Status EdBuildingWindow::WidgetActivated( Widget *widget, Window *win ) {
 ////////////////////////////////////////////////////////////////////////
 
 EdUnitWindow::EdUnitWindow( Unit &unit, Mission &mission, View *view ) :
-              Window(WIN_CENTER, view), u(unit), mission(mission), mv(0) { 
+              Window(WIN_CENTER, view), u(unit), mission(mission), mv(0) {
   // calculate dimensions
   SetSize( MIN(sfont->Width() * 50, view->Width()),
            MIN(sfont->Height() * 12, view->Height()) );
@@ -1043,7 +980,7 @@ GUI_Status EdUnitWindow::WidgetActivated( Widget *widget, Window *win ) {
 ////////////////////////////////////////////////////////////////////////
 
 EdEventsWindow::EdEventsWindow( Mission &mission, View *view ) :
-              Window(WIN_CENTER, view), mission(mission) { 
+              Window(WIN_CENTER, view), mission(mission) {
   // calculate dimensions
   SetSize( MIN(sfont->Width() * 20 + 20, view->Width()),
            MIN( MAX(mission.GetEvents().CountNodes() + 2, 6) *
@@ -1134,7 +1071,7 @@ GUI_Status EdEventsWindow::WidgetActivated( Widget *button, Window *win ) {
 ////////////////////////////////////////////////////////////////////////
 
 SelectEventWindow::SelectEventWindow( Mission &mission, View *view ) :
-                   Window(WIN_CENTER, view), mission(mission) { 
+                   Window(WIN_CENTER, view), mission(mission) {
   // calculate dimensions
   SetSize( MIN(sfont->Width() * 18 + 20, view->Width()),
            MIN( 10 * (sfont->Height() + 2) + 42, view->Height()) );
@@ -1217,7 +1154,7 @@ void SelectEventWindow::BuildTLWList( TLWList &events ) const {
 ////////////////////////////////////////////////////////////////////////
 
 EdUnitsWindow::EdUnitsWindow( Mission &mission, MapObject &container, View *view ) :
-              Window(WIN_CENTER, view), mission(mission), container(container) { 
+              Window(WIN_CENTER, view), mission(mission), container(container) {
   TLWListBuilder::BuildUnitList( mission.GetUnits(), nodes, container.Position() );
 
   // calculate dimensions
@@ -1374,8 +1311,8 @@ SelectUnitWindow::SelectUnitWindow( Mission &mission,
 
 SelectUnitWindow::SelectUnitWindow( Mission &mission, TLWList &list,
                   WidgetHook *hook, View *view ) :
-                   Window(WIN_CENTER, view), mission(mission) {
-  Init( list, hook  );
+                  Window(WIN_CENTER, view), mission(mission) {
+  Init( list, hook );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -1450,7 +1387,7 @@ const UnitType *SelectUnitWindow::Selected( void ) const {
 ////////////////////////////////////////////////////////////////////////
 
 EdMsgWindow::EdMsgWindow( Locale &locale, View *view ) :
-     Window(WIN_CENTER, view), locale(locale), cur_lang(0), cur_msg_id(-1) { 
+     Window(WIN_CENTER, view), locale(locale), cur_lang(0), cur_msg_id(-1) {
   // calculate dimensions
   SetSize( MIN(sfont->Width() * 35 + 20, view->Width()),
            MIN((sfont->Height() + 2) * 10 + 50, view->Height()) );
