@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2005 Jens Granseuer
+// Copyright (C) 2002-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,15 +25,10 @@
 
 #include "window.h"
 #include "button.h"
-#include "edwidget.h"
 #include "misc.h"
 #include "mission.h"
 #include "mapview.h"
 
-// non-private widget ID's for various windows
-#define B_ID_NMW_CREATE  101
-#define B_ID_SUW_OK      102
-
 class ListExchangeWindow : public Window, public WidgetHook {
 public:
   ListExchangeWindow( TLWList &l1, TLWList &l2, const char *label1,
@@ -43,9 +38,11 @@ public:
 private:
   GUI_Status WidgetActivated( Widget *widget, Window *win );
 
-  static const short B_ID_OK;
-  static const short B_ID_RIGHT;
-  static const short B_ID_LEFT;
+  enum {
+    B_ID_OK = 1,
+    B_ID_RIGHT,
+    B_ID_LEFT
+  };
 
   ButtonWidget *ok;
   TextListWidget *wd_l1;
@@ -78,7 +75,8 @@ class NewMissionWindow : public Window, public WidgetHook {
 public:
   NewMissionWindow( View *view );
 
-  void SetHook( WidgetHook *hook ) { this->hook = hook; }
+  void SetHook( WidgetHook *hook, short id )
+    { this->hook = hook; ok_id = id; }
 
   Mission *GetMission( void ) const { return mission; }
 
@@ -91,11 +89,14 @@ private:
 
   GUI_Status WidgetActivated( Widget *button, Window *win );
 
-  static const short B_ID_UNITSET;
-  static const short B_ID_TILESET;
-  static const short B_ID_UNITSET_OK;
-  static const short B_ID_TILESET_OK;
-  static const short B_ID_RANDOM;
+  enum {
+    B_ID_UNITSET = 1,
+    B_ID_TILESET,
+    B_ID_UNITSET_OK,
+    B_ID_TILESET_OK,
+    B_ID_RANDOM,
+    B_ID_OK
+  };
 
   Mission *mission;
 
@@ -107,6 +108,7 @@ private:
   SliderWidget *gen_water;
   SliderWidget *gen_roughness;
   WidgetHook *hook;
+  short ok_id;
 };
 
 class EdMissionSetupWindow : public Window, public WidgetHook {
@@ -116,21 +118,23 @@ public:
 private:
   GUI_Status WidgetActivated( Widget *widget, Window *win );
 
-  static const short B_ID_NAME;
-  static const short B_ID_PLAYER1;
-  static const short B_ID_PLAYER2;
-  static const short B_ID_BRIEF1;
-  static const short B_ID_BRIEF2;
-  static const short B_ID_INFO;
-  static const short B_ID_CAMPAIGN;
-  static const short B_ID_SKIRMISH;
-  static const short B_ID_CAMPAIGN_NAME;
-  static const short B_ID_CAMPAIGN_INFO;
-  static const short B_ID_SHOWMSG;
-  static const short S_ID_SEQUEL;
-  static const short B_ID_PLAYERS;
-  static const short B_ID_SET;
-  static const short S_ID_MUSIC;
+  enum {
+    B_ID_NAME,
+    B_ID_INFO,
+    B_ID_PLAYER1,
+    B_ID_PLAYER2,
+    B_ID_BRIEF1,
+    B_ID_BRIEF2,
+    B_ID_CAMPAIGN_NAME,
+    B_ID_CAMPAIGN_INFO,
+    B_ID_SHOWMSG,
+    B_ID_SET,
+    S_ID_SEQUEL,
+    B_ID_PLAYERS,
+    S_ID_MUSIC,
+    B_ID_CAMPAIGN,
+    B_ID_SKIRMISH
+  };
 
   Mission &mission;
 
@@ -148,16 +152,18 @@ public:
 private:
   GUI_Status WidgetActivated( Widget *widget, Window *win );
 
-  static const short B_ID_PLAYER;
-  static const short B_ID_WORKSHOP;
-  static const short B_ID_FACTORY;
-  static const short B_ID_CRYSTALS;
-  static const short B_ID_OK;
-  static const short B_ID_NAME;
-  static const short B_ID_NAME_OK;
-  static const short B_ID_UNITS;
-  static const short N_ID_MAXCRYSTALS;
-  static const short B_ID_FACTORY_UNITS;
+  enum {
+    B_ID_PLAYER = 1,
+    B_ID_WORKSHOP,
+    B_ID_FACTORY,
+    B_ID_CRYSTALS,
+    B_ID_OK,
+    B_ID_NAME,
+    B_ID_NAME_OK,
+    B_ID_UNITS,
+    N_ID_MAXCRYSTALS,
+    B_ID_FACTORY_UNITS
+  };
 
   Building &b;
   Mission &mission;
@@ -184,13 +190,15 @@ public:
 private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
 
-  static const short B_ID_PLAYER;
-  static const short B_ID_DIRECTION;
-  static const short N_ID_CRYSTALS;
-  static const short B_ID_UNITS;
-  static const short B_ID_SIZE;
-  static const short B_ID_XP;
-  static const short B_ID_OK;
+  enum {
+    B_ID_PLAYER = 1,
+    B_ID_DIRECTION,
+    B_ID_SIZE,
+    B_ID_XP,
+    N_ID_CRYSTALS,
+    B_ID_UNITS,
+    B_ID_OK
+  };
 
   Unit &u;
   Mission &mission;
@@ -207,10 +215,12 @@ public:
 private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
 
-  static const short B_ID_NEW;
-  static const short B_ID_EDIT;
-  static const short B_ID_DELETE;
-  static const short B_ID_OK;
+  enum {
+    B_ID_NEW = 1,
+    B_ID_EDIT,
+    B_ID_DELETE,
+    B_ID_OK
+  };
 
   TextListWidget *e_list;
   TLWList nodes;
@@ -226,9 +236,11 @@ private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
   void BuildTLWList( TLWList &list ) const;
 
-  static const short B_ID_TRIGGER;
-  static const short B_ID_OK;
-  static const short B_ID_CANCEL;
+  enum {
+    B_ID_TRIGGER = 1,
+    B_ID_OK,
+    B_ID_CANCEL
+  };
 
   TextListWidget *e_list;
   CycleWidget *e_trigger;
@@ -244,10 +256,12 @@ private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
   void BuildTLWList( TLWList &list ) const;
 
-  static const short B_ID_NEW;
-  static const short B_ID_EDIT;
-  static const short B_ID_DELETE;
-  static const short B_ID_OK;
+  enum {
+    B_ID_NEW = 1,
+    B_ID_EDIT,
+    B_ID_DELETE,
+    B_ID_OK
+  };
 
   TextListWidget *e_list;
   TLWList nodes;
@@ -268,7 +282,9 @@ private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
   void Init( TLWList &list, WidgetHook *hook );
 
-  static const short B_ID_CANCEL;
+  enum {
+    B_ID_CANCEL = 1
+  };
 
   TextListWidget *e_list;
   TLWList nodes;
@@ -285,12 +301,14 @@ private:
   void BuildMsgsList( void );
   GUI_Status WidgetActivated( Widget *button, Window *win );
 
-  static const short B_ID_LANG_NEW;
-  static const short B_ID_LANG_DELETE;
-  static const short B_ID_MSG_ADD;
-  static const short B_ID_MSG_MODIFY;
-  static const short L_ID_LANGS;
-  static const short L_ID_MSGS;
+  enum {
+    B_ID_LANG_NEW = 1,
+    B_ID_LANG_DELETE,
+    B_ID_MSG_ADD,
+    B_ID_MSG_MODIFY,
+    L_ID_LANGS,
+    L_ID_MSGS
+  };
 
   TextListWidget *w_langs;
   TextListWidget *w_msgs;
@@ -304,5 +322,5 @@ private:
   int cur_msg_id;
 };
 
-#endif	// _INCLUDE_ED_EXTWINDOW2_H
+#endif  // _INCLUDE_ED_EXTWINDOW2_H
 
