@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2005 Jens Granseuer
+// Copyright (C) 2002-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,7 +29,8 @@
 
 const char *event_labels[] = {
       "Message", "Mining", "Score", "Configure", "Create Unit",
-      "Manipulate Event", "Research", "Set Hex", "Set Timer", 0 };
+      "Manipulate Event", "Research", "Set Hex", "Set Timer",
+      "Destroy Unit", 0 };
 const char *etrigger_labels[] = {
       "Timer", "Unit Destruction", "Shop Owner", "Unit Owner",
       "Unit Position", "Handicap", "Have Crystals", 0 };
@@ -942,6 +943,13 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
   case EVENT_MESSAGE:
   case EVENT_SET_HEX:
     break;
+  case EVENT_DESTROY_UNIT:
+    if ((e.GetData(0) == -1) || !GetUnitByID(e.GetData(0))) {
+      ++errs;
+      errors << "Error: Invalid unit " << e.GetData(0) << " for "
+             << e.Name() << " (" << (short)e.ID() << ") event.\n\n";
+    }
+    break;
   case EVENT_SCORE:
     if ( (e.GetData(1) != -1) && !GetMessage(e.GetData(1)) ) {
       ++errs;
@@ -1034,7 +1042,7 @@ Player::Player( unsigned char id ) :
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Player::Load( File &file ) {
+int Player::Load( MemBuffer &file ) {
   p_id = file.Read8();
   p_type = file.Read8();
   p_success = file.Read8();
@@ -1060,7 +1068,7 @@ int Player::Load( File &file ) {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int Player::Save( File &file ) const {
+int Player::Save( MemBuffer &file ) const {
   file.Write8( p_id );
   file.Write8( p_type );
   file.Write8( p_success );
@@ -1129,6 +1137,11 @@ Event::Event( unsigned char id, unsigned char type, unsigned char trigger ) {
     e_data[2] = NORTH | (MAX_GROUP_SIZE << 3) | (0 << 6);
                           // face northwards, MAX_GROUP_SIZE, no xp
     break;
+  case EVENT_DESTROY_UNIT:
+    e_data[0] = -1;       // destroy unit -1
+    e_data[1] = -1;        // any owner
+    e_data[2] = 0;
+    break;
   case EVENT_MANIPULATE_EVENT:
     e_data[0] = -1;      // modify event -1
     e_data[1] = 0;       // flags affected
@@ -1206,7 +1219,7 @@ Event::Event( unsigned char id, unsigned char type, unsigned char trigger ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Event::Load( File &file ) {
+int Event::Load( MemBuffer &file ) {
   int i;
 
   e_id = file.Read8();
@@ -1230,7 +1243,7 @@ int Event::Load( File &file ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Event::Save( File &file ) const {
+int Event::Save( MemBuffer &file ) const {
   int i;
 
   file.Write8( e_id );
@@ -1257,7 +1270,8 @@ int Event::Save( File &file ) const {
 
 int Event::Export( ofstream &file, const Map &map ) const {
   const char *type[] = { "message", "mining", "score", "configure",
-        "createunit", "manipulateevent", "research", "sethex", "settimer" };
+        "createunit", "manipulateevent", "research", "sethex", "settimer",
+        "destroyunit" };
   Point loc;
   const UnitSet *uset = map.GetUnitSet();
 
@@ -1343,6 +1357,10 @@ int Event::Export( ofstream &file, const Map &map ) const {
     if ( ((e_data[2] & 0x1C0) >> 6) != 0 )
       file << "xp = " << ((e_data[2] & 0x1C0) >> 6) << '\n';
     break;
+  case EVENT_DESTROY_UNIT:
+    file << "unit = " << e_data[0] << '\n';
+    file << "owner = " << (e_data[1] == -1 ? -1 : e_data[1] + 1) << '\n';
+    break;
   case EVENT_MANIPULATE_EVENT:
     file << "event = " << e_data[0] << '\n';
     file << "eflags = " << e_data[1] << '\n';
