@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2005 Jens Granseuer
+// Copyright (C) 2002-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -34,8 +34,8 @@ class Player {
 public:
   Player( unsigned char id );
 
-  int Load( File &file );
-  int Save( File &file ) const;
+  int Load( MemBuffer &file );
+  int Save( MemBuffer &file ) const;
   int Export( ofstream &file ) const;
 
   void SetID( unsigned char id ) { p_id = id; }
@@ -66,8 +66,8 @@ public:
   Event( void ) {}
   Event( unsigned char id, unsigned char type, unsigned char trigger );
 
-  int Load( File &file );
-  int Save( File &file ) const;
+  int Load( MemBuffer &file );
+  int Save( MemBuffer &file ) const;
   int Export( ofstream &file, const Map &map ) const;
 
   unsigned char ID( void ) const { return e_id; }
