@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2005 Jens Granseuer
+// Copyright (C) 2002-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@ Unit::Unit( const UnitType *type, unsigned char pid, unsigned short id, const Po
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-Unit::Unit( File &file, const UnitType *type ) :
+Unit::Unit( MemBuffer &file, const UnitType *type ) :
       MapObject( MO_UNIT ) {
   u_pid = file.Read8();
   u_pos.x = file.Read16();
@@ -84,7 +84,7 @@ Unit::Unit( File &file, const UnitType *type ) :
 // RETURNS    : 0 on succes, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int Unit::Save( File &file ) const {
+int Unit::Save( MemBuffer &file ) const {
   file.Write8( u_type->ID() );
   file.Write8( u_pid );
 
