@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2005 Jens Granseuer
+// Copyright (C) 2002-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -57,9 +57,9 @@ class Unit : public Node, public MapObject {
 public:
   Unit( void ) : MapObject(MO_UNIT) {}
   Unit( const UnitType *type, unsigned char pid, unsigned short id, const Point &pos );
-  Unit( File &file, const UnitType *type );
+  Unit( MemBuffer &file, const UnitType *type );
 
-  int Save( File &file ) const;
+  int Save( MemBuffer &file ) const;
   int Export( ofstream &file ) const;
 
   unsigned short BaseImage( void ) const { return u_type->Image() + u_pid * 6; }
