@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "misc.h"
 #include "sound.h"
 #include "msgs.h"
+#include "globals.h"  // for MIN_XRES, MIN_YRES
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : DialogWindow::DialogWindow
@@ -73,8 +74,8 @@ DialogWindow::DialogWindow( const char *title, const string &msg,
   if ( prefw > view->Width() ) prefw = view->Width();
 
   Rect win( 0, 0, prefw,
-            sfont->TextHeight(msg.c_str(), prefw-20, 2) + 
-            sfont->Height() + 
+            sfont->TextHeight(msg.c_str(), prefw-20, 2) +
+            sfont->Height() +
             (title ? tfont->Height() + 10 : 5) + 20 );
   win.Clip( *view );
 
@@ -186,7 +187,7 @@ MessageWindow::MessageWindow( const char *title, const char *msg, View *view ) :
   this->title = title;
 
   // calculate window dimensions
-  SetSize( MAX(view->Width()/2, 320), MAX(view->Height()*3/4, 240) );
+  SetSize( MAX(view->Width()/2, MIN_XRES), MAX(view->Height()*3/4, MIN_YRES) );
 
   button = new ButtonWidget( GUI_CLOSE, 1, h - sfont->Height() - 11, w - 2, sfont->Height() + 10,
                              WIDGET_DEFAULT, MSG(MSG_B_OK), this );
@@ -328,6 +329,7 @@ void PasswordWindow::NewPassword( const char *pass ) {
 //              h     - window height
 //              pmin  - value indicating an empty progress bar
 //              pmax  - value indicating a filled progress bar
+//              msg   - message to display (may be NULL)
 //              flags - window flags (see window.h for details)
 //              view  - pointer to the window's view
 // RETURNS    : -
@@ -335,7 +337,7 @@ void PasswordWindow::NewPassword( const char *pass ) {
 
 ProgressWindow::ProgressWindow( short x, short y, unsigned short w,
                 unsigned short h, short pmin, short pmax,
-                unsigned short flags, View *view ) :
+                const char *msg, unsigned short flags, View *view ) :
                 Window( x, y, w, h, flags, view ) {
   short bwidth = 0;
 
@@ -343,11 +345,12 @@ ProgressWindow::ProgressWindow( short x, short y, unsigned short w,
     const char *lbl = MSG(MSG_B_SKIP);
     bwidth = sfont->TextWidth( lbl ) + 15;
     new ButtonWidget( GUI_CLOSE, w - bwidth - 5, 5, bwidth, h - 10,
-                      WIDGET_DEFAULT, lbl, this );
+                      (flags & WIN_PROG_DEFAULT ? WIDGET_DEFAULT : 0),
+                      lbl, this );
   }
 
   progress = new ProgressWidget( GUI_OK, 5, 5, w - 10 - bwidth, h - 10,
-                                 pmin, pmax, 0, NULL, this );
+                                 pmin, pmax, 0, msg, this );
   progress->SetColor( GetFGPen() );
 
   Draw();
@@ -355,17 +358,17 @@ ProgressWindow::ProgressWindow( short x, short y, unsigned short w,
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : ProgressWindow::Aborted
-// DESCRIPTION: Check whether the user clicked the 'Abort' button.
+// NAME       : ProgressWindow::Cancelled
+// DESCRIPTION: Check whether the user clicked the 'Cancel' button.
 // PARAMETERS : -
 // RETURNS    : TRUE if button was activated, FALSE otherwise
 ////////////////////////////////////////////////////////////////////////
 
-bool ProgressWindow::Aborted( void ) {
+bool ProgressWindow::Cancelled( void ) {
   GUI_Status peek;
   bool abort = false;
 
-  do { 
+  do {
     SDL_Event event;
     peek = view->PeekEvent( event );
     if ( (peek != GUI_NONE) &&
@@ -393,7 +396,7 @@ MenuWindow::MenuWindow( const char *title, WidgetHook *hook, View *view ) :
             layout(false), position(-1,-1), item_hook(hook), parent_menu(0) {
   minwidth = 0;
   vspacing = 4;
-  barheight = vspacing * 2 + 2;		// height of the separator bar
+  barheight = vspacing * 2 + 2;    // height of the separator bar
 
   if ( title ) {
     items.AddHead( new MenuItem( MENU_TITLE, 0, 0, 0, title ) );
