@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -98,13 +98,13 @@ private:
 
 
 // progress window
-class ProgressWindow : public Window {
+class ProgressWindow : public Window, public UserActionHook {
 public:
   ProgressWindow( short x, short y, unsigned short w,
                   unsigned short h, short pmin, short pmax,
-                  unsigned short flags, View *view );
+                  const char *msg, unsigned short flags, View *view );
 
-  bool Aborted( void );
+  virtual bool Cancelled( void );
 
   void Set( short pos ) const { progress->SetLevel( pos ); }
   short Get( void ) const { return progress->Level(); }
@@ -123,13 +123,13 @@ private:
 class MenuWindow : public Window, private WidgetHook {
 public:
   MenuWindow( const char *title, WidgetHook *hook, View *view );
-                     
+
   void AddItem( unsigned char level, short id, unsigned short flags,
                 const char *title );
   void AddMenu( unsigned char level, unsigned short flags,
                 const char *title );
   void AddBar( unsigned char level );
-                          
+
   void SetPosition( const Point &pos ) { position = pos; }
   void SetMinWidth( unsigned short w ) { minwidth = w; }
   void Layout( void );
@@ -172,5 +172,5 @@ private:
   MenuWindow *parent_menu;
 };
 
-#endif	/* _INCLUDE_EXTWINDOW_H */
+#endif  /* _INCLUDE_EXTWINDOW_H */
 
