@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -267,25 +267,25 @@ bool File::Exists( const string &name ) {
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : File::Read8
-// DESCRIPTION: Read a byte from the file.
+// DESCRIPTION: Read a byte from the buffer.
 // PARAMETERS : -
 // RETURNS    : byte read
 ////////////////////////////////////////////////////////////////////////
 
-unsigned char File::Read8( void ) const {
+unsigned char File::Read8( void ) {
   unsigned char val;
   Read( &val, 1 );
   return val;
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : File::ReadS
-// DESCRIPTION: Read a string from the file.
+// NAME       : MemBuffer::ReadS
+// DESCRIPTION: Read a string from the buffer.
 // PARAMETERS : size - number of bytes to read
 // RETURNS    : string read
 ////////////////////////////////////////////////////////////////////////
 
-string File::ReadS( int size ) const {
+string MemBuffer::ReadS( int size ) {
   string str;
 
   for ( int i = 0; i < size; ++i )
@@ -295,21 +295,22 @@ string File::ReadS( int size ) const {
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : File::WriteS
-// DESCRIPTION: Read a string to the file.
+// NAME       : MemBuffer::WriteS
+// DESCRIPTION: Write a string to the buffer.
 // PARAMETERS : value - string to write
 //              len   - minimum number of bytes to write. If value is
 //                      shorter than len, a number of NUL-bytes are
-//                      written to the file.
+//                      written to the buffer.
 // RETURNS    : -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int File::WriteS( string value, int len /* = 0 */ ) const {
+int MemBuffer::WriteS( string value, int len /* = 0 */ ) {
   int size = value.size();
-  int rc = SDL_RWwrite(fh, value.c_str(), value.size(), 1);
+  int rc = Write( value.c_str(), size );
 
-  if ( size < len ) {
-    for ( ; len > size; --len ) Write8( 0 );
+  while ( (size < len) && (rc == 0) ) {
+    rc = Write8( 0 );
+    ++size;
   }
   return rc;
 }
@@ -341,7 +342,7 @@ void make_dir( const char *dir ) {
   size_t len = mbstowcs( wdir, dir, strlen(dir) );
   if ( len < 0 ) len = 0;
   wdir[len] = 0;
-  CreateDirectory( wdir, NULL );                         
+  CreateDirectory( wdir, NULL );
   delete [] wdir;
 # else
   CreateDirectory( dir, NULL );
