@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #define GI_CAMPAIGN      0x0004   /* campaign map */
 #define GI_AI            0x0008   /* intended as a single-player map */
 #define GI_SKIRMISH      0x0010   /* map is suitable for skirmish play */
+#define GI_NETWORK       0x0020   /* network game */
 #define GI_GAME_OVER     0x8000
 
 #define TURN_START		0
@@ -57,6 +58,7 @@
 #define EVENT_RESEARCH		7  /* enable new units in factories */
 #define EVENT_SET_HEX		8  /* change a map tile */
 #define EVENT_SET_TIMER		9  /* adjust the timer for another event */
+#define EVENT_DESTROY_UNIT	10 /* remove unit from board */
 
 /* event triggers */
 #define ETRIGGER_TIMER          0
