@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -32,17 +32,20 @@
  */
 
 #ifndef VERSION
-# define VERSION "0.4.9"
+# define VERSION "0.5.0"
 #endif
 
 #define CF_SHORTNAME	"crimson"
 #define CF_DATFILE	"cf.dat"
-#define CF_TITLE_SCREEN	"title.bmp"
 
 #define CF_FONT		"Bepa-Roman.ttf"
 #define CF_FONT_SMALL	12
 #define CF_FONT_LARGE	16
 
+/* font sizes for small resolutions */
+#define CF_FONT_LOWRES_SMALL	9
+#define CF_FONT_LOWRES_LARGE	11
+
 #define CF_MUSIC_THEME		"theme"
 #define CF_MUSIC_DEFAULT	"default"
 #define CF_MUSIC_FADE_TIME	2000
@@ -52,10 +55,18 @@
 
 #define DISPLAY_BPP	16	/* display depth */
 
-#define MIN_XRES	320
+#define MIN_XRES	240
 #define MIN_YRES	240
 
-#ifdef _WIN32_WCE
+#if defined PSP || defined V43
+# define DEFAULT_RESOLUTION	480,272
+#elif defined _WIN32_WCE
+# define DEFAULT_RESOLUTION	240,320
+#else
+# define DEFAULT_RESOLUTION	800,600
+#endif
+
+#if defined _WIN32_WCE || defined PSP
 # define strcasecmp _stricmp
 # define strncasecmp _strnicmp
 #else
