@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ int Language::ReadCatalog( const char *catalog ) {
 // RETURNS    : number of messages read (>= 0) on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Language::ReadCatalog( File &file ) {
+int Language::ReadCatalog( MemBuffer &file ) {
   int rc;
 
   // dump old messages
@@ -114,7 +114,7 @@ int Language::WriteCatalog( const char *catalog ) {
 // RETURNS    : number of messages written on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Language::WriteCatalog( File &file ) {
+int Language::WriteCatalog( MemBuffer &file ) {
   file.WriteS( id );
 
   // calculate catalog size
@@ -167,7 +167,7 @@ short Language::Find( const string &msg ) const {
 // RETURNS    : number of languages read (>= 0) on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Locale::Load( File &file ) {
+int Locale::Load( MemBuffer &file ) {
 
   unsigned short langs = file.Read16();
   for ( int i = 0; i < langs; ++i ) {
@@ -195,7 +195,7 @@ int Locale::Load( File &file ) {
 // RETURNS    : number of languages written on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Locale::Save( File &file ) {
+int Locale::Save( MemBuffer &file ) {
   file.Write16( lib.size() );
 
   for ( map<const string, Language>::iterator iter = lib.begin();
