@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,9 +39,9 @@
 class Language {
 public:
   int ReadCatalog( const char *catalog );
-  int ReadCatalog( File &file );
+  int ReadCatalog( MemBuffer &file );
   int WriteCatalog( const char *catalog );
-  int WriteCatalog( File &file );
+  int WriteCatalog( MemBuffer &file );
 
   const char *ID( void ) const { return id.c_str(); }
   const char *Name( void ) const { return name.c_str(); }
@@ -66,8 +66,8 @@ class Locale {
 public:
   Locale( void ) : lang(0) {}
 
-  int Load( File &file );
-  int Save( File &file );
+  int Load( MemBuffer &file );
+  int Save( MemBuffer &file );
 
   void AddLanguage( Language &lang );
   void RemoveLanguage( Language &lang );
