@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int TileSet::Load( File &file, const char *setname ) {
+int TileSet::Load( MemBuffer &file, const char *setname ) {
 
   int rc = tiles.LoadImageData( file, true );
   if ( !rc ) {
@@ -147,7 +147,7 @@ SoundEffect *UnitSet::GetSound( unsigned short sfxid ) const {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int UnitSet::Load( File &file, const char *setname ) {
+int UnitSet::Load( MemBuffer &file, const char *setname ) {
   int rc = -1;
 
   if ( file.Read32() == FID_UNITSET ) {
@@ -180,7 +180,7 @@ int UnitSet::Load( File &file, const char *setname ) {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int UnitSet::LoadUnitTypes( File &file ) {
+int UnitSet::LoadUnitTypes( MemBuffer &file ) {
   ut = new UnitType [num_tiles];
   if ( !ut ) return -1;
 
@@ -215,7 +215,7 @@ int UnitSet::LoadUnitTypes( File &file ) {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int UnitSet::LoadSfx( File &file ) {
+int UnitSet::LoadSfx( MemBuffer &file ) {
   if ( num_sfx ) {
     string sfxd( get_sfx_dir() ), sfxname;
     unsigned char len;
@@ -255,7 +255,7 @@ void UnitSet::SetLocale( const string &lang ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int TerrainType::Load( File &file ) {
+int TerrainType::Load( MemBuffer &file ) {
   tt_type = file.Read16();
   tt_image = file.Read16();
   tt_att_mod = file.Read8();
@@ -287,7 +287,7 @@ TerrainSet::~TerrainSet( void ) {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int TerrainSet::Load( File &file, const char *setname ) {
+int TerrainSet::Load( MemBuffer &file, const char *setname ) {
   int rc = -1;
 
   if ( file.Read32() == FID_TERRAINSET ) {
@@ -332,7 +332,7 @@ const TerrainType *TerrainSet::GetTerrainInfo( unsigned short ttid ) const {
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
-int TerrainSet::LoadTerrainTypes( File &file ) {
+int TerrainSet::LoadTerrainTypes( MemBuffer &file ) {
   tt = new TerrainType [num_tiles];
   if ( !tt ) return -1;
 
@@ -369,7 +369,7 @@ void TerrainSet::DrawFog( Surface *dest, short px, short py, const Rect &clip )
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int UnitType::Load( File &file, const UnitSet *set ) {
+int UnitType::Load( MemBuffer &file, const UnitSet *set ) {
   ut_snd_move = ut_snd_fire = NULL;
   ut_terrain = file.Read16();
   ut_image = file.Read16();
