@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
 // terrain type definitions
 class TerrainType {
 public:
-  int Load( File &file );
+  int Load( MemBuffer &file );
 
   unsigned short tt_type;       // one (or more) of the TT_ defined above
   unsigned short tt_image;
@@ -54,7 +54,7 @@ class UnitSet;
 class UnitType {
 public:
   UnitType( void ) {}
-  int Load( File &file, const UnitSet *set );
+  int Load( MemBuffer &file, const UnitSet *set );
   void SetName( const char *name ) { ut_name = name; }
 
   unsigned short Terrain( void ) const { return ut_terrain; }
@@ -114,7 +114,7 @@ public:
   TileSet( void ) : num_tiles(0) {}
   virtual ~TileSet( void ) {}
 
-  virtual int Load( File &file, const char *setname );
+  virtual int Load( MemBuffer &file, const char *setname );
 
   unsigned short TileWidth( void ) const { return DEFAULT_TILE_WIDTH; }
   unsigned short TileHeight( void ) const { return DEFAULT_TILE_HEIGHT; }
@@ -137,7 +137,7 @@ public:
   UnitSet( void );
   ~UnitSet( void );
 
-  int Load( File &file, const char *setname );
+  int Load( MemBuffer &file, const char *setname );
 
   const UnitType *GetUnitInfo( unsigned short utid ) const;
   SoundEffect *GetSound( unsigned short sfxid ) const;
@@ -148,8 +148,8 @@ public:
   void SetLocale( const string &lang );
 
 protected:
-  int LoadUnitTypes( File &file );
-  int LoadSfx( File &file );
+  int LoadUnitTypes( MemBuffer &file );
+  int LoadSfx( MemBuffer &file );
 
   unsigned short num_sfx;
 
@@ -165,7 +165,7 @@ public:
   TerrainSet( void ) : tt(0) {}
   ~TerrainSet( void );
 
-  int Load( File &file, const char *setname );
+  int Load( MemBuffer &file, const char *setname );
 
   unsigned short GetTerrainClassID( unsigned char num ) const { return classid[num]; }
   const TerrainType *GetTerrainInfo( unsigned short ttid ) const;
@@ -174,7 +174,7 @@ public:
   const Surface &HexMask( void ) const { return fog; }
 
 protected:
-  int LoadTerrainTypes( File &file );
+  int LoadTerrainTypes( MemBuffer &file );
 
   TerrainType *tt;
   Surface fog;
