@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -180,7 +180,7 @@ void MapView::DrawMap( short x, short y, unsigned short w,
 
         // draw unit health
         if ( UnitStatsEnabled() )
-          DrawUnitHealth( u->GroupSize(), surface, sx, sy, clip );
+          DrawUnitHealth( u->GroupSize(), dest, sx, sy, clip );
       }
 
       // draw fog
@@ -332,8 +332,8 @@ void MapView::DrawUnitHealth( unsigned char health, Surface *dest,
   outer.Clip( clip );
   inner.Clip( clip );
 
-  surface->FillRect( outer, Color(CF_COLOR_BLACK) );
-  surface->FillRect( inner, hcol );
+  dest->FillRect( outer, Color(CF_COLOR_BLACK) );
+  dest->FillRect( inner, hcol );
 }
 
 ////////////////////////////////////////////////////////////////////////
