@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,8 +31,8 @@
 //              y      - top edge of widget
 //              w      - widget width
 //              h      - widget height
-//              min    - lowest slider level allowed
-//              max    - highest slider level allowed
+//              vmin   - lowest slider level allowed
+//              vmax   - highest slider level allowed
 //              start  - default level
 //              ksize  - size of the slider knob in items (if the slider
 //                       has 10 possible positions and ksize is 2, the
@@ -47,14 +47,14 @@
 ////////////////////////////////////////////////////////////////////////
 
 SliderWidget::SliderWidget( short id, short x, short y, unsigned short w,
-    unsigned short h, short min, short max, short start, short ksize,
+    unsigned short h, short vmin, short vmax, short start, short ksize,
     unsigned short flags, const char *title, Window *window ) :
     Widget( id, x, y, w, h, flags, title, window ) {
   mousehit = false;
   current = start;
   keystep = 1;
 
-  Adjust( min, max, ksize );
+  Adjust( vmin, vmax, ksize );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -67,28 +67,28 @@ SliderWidget::SliderWidget( short id, short x, short y, unsigned short w,
 ////////////////////////////////////////////////////////////////////////
 
 void SliderWidget::Adjust( short newmin, short newmax, short newsize ) {
-  min = newmin;
-  max = newmax;
+  min_level = newmin;
+  max_level = newmax;
   size = MAX( newsize, 1 );
 
-  if ( current < min ) current = min;
-  else if ( current > max ) current = max;
+  if ( current < min_level ) current = min_level;
+  else if ( current > max_level ) current = max_level;
 
-  short num = max - min;
+  short num = max_level - min_level;
   if ( flags & WIDGET_HSCROLL ) {
-    knob.w = MAX( 2, (w - 2) * size / (size + num) ); 
+    knob.w = MAX( 2, (w - 2) * size / (size + num) );
     knob.h = h - 2;
-    if ( max > min ) step = (float)(w - 2 - knob.w) / num;
+    if ( max_level > min_level ) step = (float)(w - 2 - knob.w) / num;
     else step = 0;
-    knob.x = (short)(x + 1 + (current - min) * step + 0.5);
+    knob.x = (short)(x + 1 + (current - min_level) * step + 0.5);
     knob.y = y + 1;
   } else {
     knob.w = w - 2;
     knob.h = MAX( 2, (h - 2) * size / (size + num) );
-    if ( max > min ) step = (float)(h - 2 - knob.h) / num;
+    if ( max_level > min_level ) step = (float)(h - 2 - knob.h) / num;
     else step = 0;
     knob.x = x + 1;
-    knob.y = (short)(y + 1 + (current - min) * step + 0.5);
+    knob.y = (short)(y + 1 + (current - min_level) * step + 0.5);
   }
 }
 
@@ -120,17 +120,17 @@ void SliderWidget::Draw( void ) {
 
 void SliderWidget::ScrollTo( short level ) {
   current = level;
-  if ( current < min ) current = min;
-  else if ( current > max ) current = max;
+  if ( current < min_level ) current = min_level;
+  else if ( current > max_level ) current = max_level;
 
   if ( flags & WIDGET_HSCROLL )
-    knob.x = (short)(x + 1 + (current - min) * step + 0.5);
-  else knob.y = (short)(y + 1 + (current - min) * step + 0.5);
+    knob.x = (short)(x + 1 + (current - min_level) * step + 0.5);
+  else knob.y = (short)(y + 1 + (current - min_level) * step + 0.5);
 
   if ( hook ) hook->WidgetActivated( this, surface );
   Draw();
   Show();
-} 
+}
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : SliderWidget::MouseDown
@@ -196,31 +196,31 @@ GUI_Status SliderWidget::KeyDown( const SDL_keysym &key ) {
   if ( key.sym == this->key ) {
     Push();
     if ( key.mod & (KMOD_LSHIFT|KMOD_RSHIFT) ) {
-      if ( current > min ) ScrollTo( current - keystep );
-    } else if ( current < max ) ScrollTo( current + keystep );
+      if ( current > min_level ) ScrollTo( current - keystep );
+    } else if ( current < max_level ) ScrollTo( current + keystep );
   } else if ( flags & (WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY) ) {
     switch( key.sym ) {
     case SDLK_UP:
-      if ( (flags & WIDGET_VSCROLLKEY) && (current > min) ) {
+      if ( (flags & WIDGET_VSCROLLKEY) && (current > min_level) ) {
         Push();
         ScrollTo( current - keystep );
       }
       break;
     case SDLK_DOWN:
-      if ( (flags & WIDGET_VSCROLLKEY) && (current < max) ) {
+      if ( (flags & WIDGET_VSCROLLKEY) && (current < max_level) ) {
         Push();
         ScrollTo( current + keystep );
       }
       break;
 
     case SDLK_LEFT:
-      if ( (flags & WIDGET_HSCROLLKEY) && (current > min) ) {
+      if ( (flags & WIDGET_HSCROLLKEY) && (current > min_level) ) {
         Push();
         ScrollTo( current - keystep );
       }
       break;
     case SDLK_RIGHT:
-      if ( (flags & WIDGET_HSCROLLKEY) && (current < max) ) {
+      if ( (flags & WIDGET_HSCROLLKEY) && (current < max_level) ) {
         Push();
         ScrollTo( current + keystep );
       }
@@ -255,23 +255,23 @@ GUI_Status SliderWidget::KeyUp( const SDL_keysym &key ) {
 GUI_Status SliderWidget::MouseMove( const SDL_MouseMotionEvent &motion ) {
   if ( mousehit ) {
 
-    if ( max != min ) {
+    if ( max_level != min_level ) {
       short val;
 
       if ( flags & WIDGET_HSCROLL ) {
         short dx = knob.x + motion.xrel;
         knob.x = MIN( MAX( x + 1, dx ), x + w - 1 - knob.w );
-        val = (short)((knob.x - 1 - x + step/2) / step) + min;
+        val = (short)((knob.x - 1 - x + step/2) / step) + min_level;
       } else {
         short dy = knob.y + motion.yrel;
         knob.y = MIN( MAX( y + 1, dy ), y + h - 1 - knob.h );
-        val = (short)((knob.y - 1 - y + step/2) / step) + min;
+        val = (short)((knob.y - 1 - y + step/2) / step) + min_level;
       }
 
       Draw();
       Show();
       if ( val != current ) {
-        current = MAX( min, MIN( val, max ) );
+        current = MAX( min_level, MIN( val, max_level ) );
         if ( hook ) hook->WidgetActivated( this, surface );
       }
     }
@@ -289,22 +289,24 @@ GUI_Status SliderWidget::MouseMove( const SDL_MouseMotionEvent &motion ) {
 //              y      - top edge of widget
 //              w      - widget width
 //              h      - widget height
-//              min    - value mapped to an empty bar
-//              max    - value mapped to a completely filled bar
+//              vmin   - value mapped to an empty bar
+//              vmax   - value mapped to a completely filled bar
 //              flags  - widget flags (see widget.h for details)
-//              title  - widget title (currently unused)
+//              title  - widget title
 //              window - window to add this widget to
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
 ProgressWidget::ProgressWidget( short id, short x, short y, unsigned short w,
-    unsigned short h, short min, short max, unsigned short flags,
+    unsigned short h, short vmin, short vmax, unsigned short flags,
     const char *title, Window *window ) :
-    Widget( id, x, y, w, h, flags, title, window ) {
-  this->min = min;
-  this->max = max;
-  level = 0;
-  col = Color(CF_COLOR_HIGHLIGHT);
+    Widget( id, x, y, w, h, flags, title, window ),
+    level(0), min_level(vmin), max_level(vmax), col(CF_COLOR_HIGHLIGHT),
+    title_x(0), title_y(0) {
+  if ( title ) {
+    title_x = x + (w - font->TextWidth( title )) / 2;
+    title_y = y + (h - font->Height()) / 2;
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -317,10 +319,10 @@ ProgressWidget::ProgressWidget( short id, short x, short y, unsigned short w,
 ////////////////////////////////////////////////////////////////////////
 
 void ProgressWidget::SetLevel( short lev ) {
-  if ( lev < min ) lev = min;
-  else if ( lev > max ) lev = max;
+  if ( lev < min_level ) lev = min_level;
+  else if ( lev > max_level ) lev = max_level;
 
-  level = lev - min;
+  level = lev - min_level;
   Draw();
   Show();
 }
@@ -337,8 +339,9 @@ void ProgressWidget::Draw( void ) {
   surface->DrawBox( *this, BOX_RECESSED );
 
   if ( !Disabled() && (level > 0) ) {
-    unsigned short levsize = level * (Width() - 2) / (max - min);
+    unsigned short levsize = level * (Width() - 2) / (max_level - min_level);
     surface->FillRectAlpha( x + 1, y + 1, levsize, h - 2, col );
   }
-}
 
+  PrintTitle( title_x, title_y, col );
+}
