@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -33,12 +33,12 @@
 # define SDL_BUTTON_WHEELDOWN 5
 #endif
 
-#define DEFAULT_SLIDER_SIZE	12
+#define DEFAULT_SLIDER_SIZE  12
 
 class SliderWidget : public Widget {
 public:
   SliderWidget( short id, short x, short y, unsigned short w,
-    unsigned short h, short min, short max, short start,
+    unsigned short h, short vmin, short vmax, short start,
     short ksize, unsigned short flags, const char *title,
     Window *window );
 
@@ -58,8 +58,8 @@ public:
 private:
   Rect knob;              // the slider knob
 
-  short min;
-  short max;
+  short min_level;
+  short max_level;
   short current;
 
   short size;
@@ -72,10 +72,10 @@ private:
 class ProgressWidget : public Widget {
 public:
   ProgressWidget( short id, short x, short y, unsigned short w,
-               unsigned short h, short min, short max,
+               unsigned short h, short vmin, short vmax,
                unsigned short flags, const char *title, Window *window );
   void SetColor( const Color &col ) { this->col = col; }
-  short Level( void ) const { return level + min; }
+  short Level( void ) const { return level + min_level; }
   void SetLevel( short lev );
   void Advance( short diff ) { SetLevel( Level() + diff ); }
 
@@ -83,11 +83,14 @@ public:
 
 private:
   unsigned short level;
-  short min;
-  short max;
+  short min_level;
+  short max_level;
 
   Color col;
+
+  short title_x;
+  short title_y;
 };
 
-#endif	/* _INCLUDE_SLIDER_H */
+#endif  /* _INCLUDE_SLIDER_H */
 
