@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,15 +28,6 @@
 #include "fileio.h"
 
 // initialize static variables
-const int Audio::SND_GUI_ERROR       = 0;
-const int Audio::SND_GUI_PRESSED     = 1;
-const int Audio::SND_GUI_MENU_SHOW   = 2;
-const int Audio::SND_GUI_ASK         = 3;
-const int Audio::SND_GAM_SELECT      = 4;
-const int Audio::SND_GAM_REPAIR      = 5;
-
-const unsigned short Audio::SFX_LOOP = 0x0001;
-
 bool Audio::sfx_state = false;
 bool Audio::music_state = false;
 int Audio::sfx_volume;
