@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -61,6 +61,27 @@ string StringUtil::strprintf( const string &format, int arg ) {
   return result;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : strprintf
+// DESCRIPTION: Replace the first occurence of %c in a printf-style
+//              format string with the given input.
+// PARAMETERS : format - printf-style formatting template
+//              arg    - value to insert
+// RETURNS    : formatted string
+////////////////////////////////////////////////////////////////////////
+
+string StringUtil::strprintf( const string &format, char arg ) {
+  string result( format );
+  size_t pos = result.find( "%c", 0 );
+
+  if ( pos != string::npos ) {
+    char buf[2] = { arg, '\0' };
+    result.replace( pos, 2, buf );
+  }
+
+  return result;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : strprintf
 // DESCRIPTION: Replace the first occurence of %s in a printf-style
