@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@
 #include "surface.h"
 #include "globals.h"
 
-const unsigned char Surface::RAW_DATA_TRANSPARENT = 0x01;
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Surface::~Surface
 // DESCRIPTION: Free the memory used by this surface.
@@ -449,7 +447,7 @@ void Surface::DrawPixel( short x, short y, const Color &col ) const {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Surface::LoadPalette( File &file, unsigned short num ) {
+int Surface::LoadPalette( MemBuffer &file, unsigned short num ) {
   unsigned char *pal, *col;
 
   // file contains num colors
@@ -479,7 +477,7 @@ int Surface::LoadPalette( File &file, unsigned short num ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Surface::LoadImageData( File &file, bool hwsurface /* = false */ ) {
+int Surface::LoadImageData( MemBuffer &file, bool hwsurface /* = false */ ) {
   unsigned char bpp, flags;
   unsigned short width, height, colors;
   Color ckey;
