@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 #include <cctype>
 #include <string.h>
+#include <stdlib.h>
 
 #include "textbox.h"
 #include "misc.h"
@@ -269,6 +270,10 @@ void TextListWidget::DrawNodes( void ) {
   short num = toprow / ItemHeight();                        // number of top node
   TLWNode *n = static_cast<TLWNode *>(list->GetNode( num ));  // top node
   short xoff = box.x, yoff = box.y + (num * ItemHeight()) - toprow;
+  Color fcol = font->GetColor();
+
+  if ( Disabled() )
+    font->SetColor( Color(CF_COLOR_GHOSTED) );
 
   surface->DrawBack( area );
 
@@ -291,6 +296,8 @@ void TextListWidget::DrawNodes( void ) {
     ++num;
     n = static_cast<TLWNode *>( n->Next() );
   }
+
+  font->SetColor( fcol );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -632,6 +639,105 @@ GUI_Status StringWidget::SetFocus( void ) {
 }
 
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : NumberWidget::NumberWidget
+// DESCRIPTION: Create a new NumberWidget.
+// PARAMETERS : id     - widget identifier
+//              x      - left edge of widget
+//              y      - top edge of widget
+//              w      - widget width
+//              h      - widget height
+//              number - initial value to display
+//              min    - minimum allowed value
+//              max    - maximum allowed value
+//              flags  - widget flags (see widget.h for details)
+//              title  - widget title
+//              window - widget parent window
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+NumberWidget::NumberWidget( short id, short x, short y, unsigned short w, unsigned short h,
+            long number, long min, long max, unsigned short flags,
+            const char *title, Window *window ) :
+    StringWidget( id, x, y, w, h, itoa(number, numbuf), strlen(itoa(max,numbuf2)),
+                  flags, title, window ) {
+  num = number;
+  minval = min;
+  maxval = max;
+  SetValidator( this );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NumberWidget::SetNumber
+// DESCRIPTION: Fill the widget with a value.
+// PARAMETERS : number - new widget value
+//              upd    - whether to update the display (default is true)
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void NumberWidget::SetNumber( long number, bool upd /* = true */ ) {
+  num = MIN( MAX( minval, number ), maxval );
+
+  SetString( itoa( num, numbuf ), upd );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NumberWidget::ValidateKey
+// DESCRIPTION: Only accept numbers for input.
+// PARAMETERS : str - string currently entered in widget (not used)
+//              key - char to be entered
+//              pos - position at which to enter the char
+// RETURNS    : TRUE if key is accepted, FALSE if refused
+////////////////////////////////////////////////////////////////////////
+
+bool NumberWidget::ValidateKey( const char *str, unsigned short key,
+                                unsigned short pos ) const {
+  return ((key >= '0') && (key <= '9')) || ((key == '-') && (pos == 0));
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NumberWidget::SetMin
+// DESCRIPTION: Set a new minimum value.
+// PARAMETERS : min - new minimum
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void NumberWidget::SetMin( long min ) {
+  minval = min;
+
+  if ( Number() < min ) SetNumber( min );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NumberWidget::SetMax
+// DESCRIPTION: Set a new maximum value.
+// PARAMETERS : max - new maximum
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void NumberWidget::SetMax( long max ) {
+  maxval = max;
+
+  if ( Number() > max ) SetNumber( max );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : NumberWidget::Release
+// DESCRIPTION: When the widget is released make sure the number which
+//              was entered is inside the requested boundaries and
+//              adjust it if necessary.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void NumberWidget::Release( void ) {
+  const char *str = String();
+  short n = (str ? atoi(str) : 0);
+  SetNumber( n );
+  StringWidget::Release();
+}
+
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : TLWNode::TLWNode
 // DESCRIPTION: Create a new TextListWidget node.
@@ -643,15 +749,15 @@ GUI_Status StringWidget::SetFocus( void ) {
 ////////////////////////////////////////////////////////////////////////
 
 TLWNode::TLWNode( const char *name, void *data, unsigned short id ) :
-         name(name), id(id), user_data(data), user_flags(0) {
+         name(name), id(id), user_data(data) {
 }
 
 TLWNode::TLWNode( const char *name, void *data ) :
-         name(name), id(0), user_data(data), user_flags(0) {
+         name(name), id(0), user_data(data) {
 }
 
 TLWNode::TLWNode( const char *name ) :
-         name(name), id(0), user_data(0), user_flags(0) {
+         name(name), id(0), user_data(0) {
 }
 
 ////////////////////////////////////////////////////////////////////////
