@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2005 Jens Granseuer
+// Copyright (C) 2000-2006 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -136,6 +136,12 @@ public:
   virtual GUI_Status WidgetActivated( Widget *widget, Window *win ) = 0;
 };
 
+// Hook class used to signal aborting in synchronous operations
+class UserActionHook {
+public:
+  virtual bool Cancelled( void ) = 0;
+};
+
 // composite widgets encapsulate several widgets but look and act like
 // a single widget to the outside
 class CompositeWidget : public Widget {
