@@ -1,7 +1,7 @@
-/* lev2bmp -- create an image for a Crimson Fields level file
-   Copyright (C) 2004 Jens Granseuer
+/* cf2bmp -- create an image for a Crimson Fields level file
+   Copyright (C) 2004-2006 Jens Granseuer
 
-   Usage: lev2bmp <file>.lev <map>.bmp
+   Usage: cf2bmp <file>.lev <map>.bmp
    The unit and tile sets must be present in the working directory.
 
    This program is free software; you can redistribute it and/or modify
@@ -25,13 +25,21 @@ using namespace std;
 
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+
 #include "SDL.h"
 
 #include "mission.h"
 #include "mapview.h"
 #include "globals.h"
 
+#ifdef _MSC_VER
+// SDL_Main linkage destroys the command line in VS8
+#undef main
+#endif
+
 int main( int argc, char *argv[] ) {
   string outmap, level;
 
