@@ -1,5 +1,5 @@
 /* cfed -- a level editor for Crimson Fields
-   Copyright (C) 2000-2005 Jens Granseuer
+   Copyright (C) 2000-2006 Jens Granseuer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +29,11 @@
 #include "strutil.h"
 #include "globals.h"
 
+#ifdef _MSC_VER
+// SDL_Main linkage destroys the command line in VS8
+#undef main
+#endif
+
 #define MAX_MAP_WIDTH	180
 #define MAX_MAP_HEIGHT	180
 
@@ -452,6 +457,7 @@ int EventHandler::ParseSection( ifstream &in, const string *opt, unsigned long &
         else if ( val == "manipulateevent" ) e->SetType(EVENT_MANIPULATE_EVENT);
         else if ( val == "sethex" ) e->SetType(EVENT_SET_HEX);
         else if ( val == "settimer" ) e->SetType(EVENT_SET_TIMER);
+        else if ( val == "destroyunit" ) e->SetType(EVENT_DESTROY_UNIT);
         else if ( val == "createunit" ) {
           e->SetType(EVENT_CREATE_UNIT);
           e->SetData(2, NORTH | (MAX_GROUP_SIZE << 3) | (0 << 6)); // dir, group size, xp defaults 
@@ -567,6 +573,11 @@ int EventHandler::ParseSection( ifstream &in, const string *opt, unsigned long &
             }
             break;
 
+          case EVENT_DESTROY_UNIT:
+            if ( key == "unit" ) e->SetData(0, StrToNum(val));         // id of unit to destroy
+            else if ( key == "owner" ) e->SetData(1, StrToNum(val)-1); // unit owner
+            break;
+
           case EVENT_MANIPULATE_EVENT:
             if ( key == "event" ) e->SetData(0, StrToNum(val));       // ID of target event
             else if ( key == "eflags" ) e->SetData(1, StrToNum(val)); // flags to modify
@@ -1437,6 +1448,26 @@ int MissionParser::check_events( void ) const {
           }
         }
         break;
+      case EVENT_DESTROY_UNIT:
+        if ( e->GetData(0) < 0 ) {
+          cerr << "Error: No unit specified for Destroy Unit event " << (short)e->ID() << endl;
+          return 1;
+        }
+
+        if ( !GetUnitByID( e->GetData(0) ) ) {
+          cerr << "Error: Unit with ID " << e->GetData(0) << " specified in event "
+               << (short)e->ID() << " does not exist" << endl;
+          return 1;
+        }
+
+        if ( e->GetData(1) < -1 ) e->SetData(1, -1);
+        else if ( e->GetData(1) > PLAYER_TWO ) {
+          cerr << "Error: Invalid unit owner for event " << (short)e->ID() << endl;
+          return 1;
+        }
+
+        e->SetData(2, 0);
+        break;
       case EVENT_MANIPULATE_EVENT:
         if ( e->GetData(0) < 0 ) {
           cerr << "Error: Event manipulation event " << (short)e->ID() << " without valid target" << endl;
