@@ -1,5 +1,5 @@
 /* Crimson Fields - a game of tactical warfare
-   Copyright (C) 2000-2005 Jens Granseuer
+   Copyright (C) 2000-2006 Jens Granseuer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 // surface class that can save image data into a data file
 class MkSurface : public Surface {
 public:
-  int SaveImageData( const string &from, File &to, bool tp );
+  int SaveImageData( const string &from, MemBuffer &to, bool tp );
   int GuessColorKey( void ) const;
 };
 
