@@ -1,5 +1,5 @@
 /* mkunitset -- create a unit set for Crimson Fields
-   Copyright (C) 2000-2005 Jens Granseuer
+   Copyright (C) 2000-2006 Jens Granseuer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +32,11 @@ using namespace std;
 #include "lset.h"
 #include "mksurface.h"
 
+#ifdef _MSC_VER
+// SDL_Main linkage destroys the command line in VS8
+#undef main
+#endif
+
 struct UnitsetHeader {
   string images;
   vector <string> sfx;
